package com.adc.da.vdaims.service.impl;

import com.adc.da.communal.Utils.DateUtils;
import com.adc.da.enums.CertificatePrintStateEnum;
import com.adc.da.enums.CertificateVehicleOrChassis;
import com.adc.da.enums.CertificateVersionNumner;
import com.adc.da.util.exception.AdcDaBaseException;
import com.adc.da.util.utils.UserUtil;
import com.adc.da.vdaims.entity.*;
import com.adc.da.util.utils.StringUtils;
import com.adc.da.vdaims.entity.CertHgzPrintEO;
import com.adc.da.vdaims.dao.CertHgzPrintEODao;
import com.adc.da.vdaims.entity.CertificationEntity;
import com.adc.da.vdaims.enums.HgzTemplateType;
import com.adc.da.vdaims.exception.BusinessException;
import com.adc.da.vdaims.service.ICertHgzPrintEOService;
import com.adc.da.vdaims.util.*;
import com.adc.da.vdaims.vo.CertUploadVehicleInfo;
import com.adc.da.vdaims.vo.StatisticsInfo;
import com.adc.da.vdaims.webservice.certificate.ArrayOfCertificateInfo;
import com.adc.da.vdaims.webservice.certificate.ArrayOfString;
import com.adc.da.vdaims.webservice.certificate.CertificateInfo;
import com.adc.da.vdaims.webservice.certificate.OperateResult;
//import com.adc.da.pdf.ConvertToPdfUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializerProvider;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Date;

/**
 * <p>
 * 服务实现类
 * </p>
 * 作者 gen
 * 日期 2021-09-08
 */
@Service
@Transactional
public class CertHgzPrintEOServiceImpl extends ServiceImpl<CertHgzPrintEODao, CertHgzPrintEO> implements ICertHgzPrintEOService {

    @Autowired
    VehicleCertEOServiceImpl vehicleCertEOService;
    @Value("${qr.wordToPdfUrl}")
    private String wordToPdfUrl;

    @Override
    public boolean finishPrint(Integer type, String id, String dywym, String jym) {
        CertHgzPrintEO certHgzPrintEO = getById(id);
        if (null == certHgzPrintEO) {
            throw new AdcDaBaseException("合格证信息不存在");
        }
        if (CertificatePrintStateEnum.REFERENCE_PRINTED.getValue().equals(type)) {
            //参考打印
            certHgzPrintEO.setHgzPrintStatus(CertificatePrintStateEnum.REFERENCE_PRINTED.getValue());
        } else if (CertificatePrintStateEnum.PRINT.getValue().equals(type)) {
            //打印
            if (CertificatePrintStateEnum.UPDATE_PRINT.getValue().equals(certHgzPrintEO.getHgzPrintStatus())) {
                certHgzPrintEO.setHgzStatus(CertificateVersionNumner.UPDATE_PRINT.getValue());
            } else if (CertificatePrintStateEnum.CANCLE_PRINT.getValue().equals(certHgzPrintEO.getHgzPrintStatus())) {
                certHgzPrintEO.setHgzStatus(CertificateVersionNumner.UPDATE_PRINT.getValue());
            }else{
                certHgzPrintEO.setHgzStatus(CertificateVersionNumner.NO_UPLOAD.getValue());
            }
            certHgzPrintEO.setVehDywym(dywym);
            certHgzPrintEO.setVehJyw(jym);
            certHgzPrintEO.setHgzPrintStatus(CertificatePrintStateEnum.PRINT.getValue());
        }
        return updateById(certHgzPrintEO);
    }

    @Override
    public String createPdfPreview(String id) {
        CertHgzPrintEO certHgzPrintEO = this.getById(id);
        if (certHgzPrintEO == null) {
            log.error("CERT_HGZ_PRINT的数据丢失：id=" + id);
            throw new BusinessException("合格证证书数据丢失");
        }
        HgzTemplateType templateType = HgzTemplateType.getByValue(certHgzPrintEO.getHgzType());

        String templatePath = ResourceUtil.getConfigByName("realpath") + templateType.getName();
        File templateFile = new File(templatePath);
        if (!templateFile.exists()) {
            log.error(templateType.getName() + "打印模板不存在");
            throw new BusinessException(templateType.getName() + "打印模板不存在");
        }

        String qyId = UserUtil.getQyId();
        String date = com.adc.da.util.utils.DateUtils.dateToString(new Date(), com.adc.da.util.utils.DateUtils.YYYY_MM_DD_EN);
        String pathSuffix = new StringBuilder().append(qyId).append("/").append("HeGeZheng/").append(date).append("/").toString();
        try {
            WordprocessingMLPackage template = Docx4jUtil.getTemplate(templatePath);
            ObjectMapper objectMapper = new ObjectMapper();
            objectMapper.getSerializerProvider().setNullValueSerializer(new JsonSerializer<Object>() {
                @Override
                public void serialize(Object value, JsonGenerator jsonGenerator, SerializerProvider serializerProvider) throws IOException {
                    jsonGenerator.writeString(StringUtils.EMPTY);
                }
            });
            HashMap<String, String> templateData = objectMapper.readValue(objectMapper.writeValueAsString(certHgzPrintEO), HashMap.class);
            // 合格证没有打印，预览不生成二维码
            templateData.put("qr", StringUtils.EMPTY);
            // 底盘类别固定值”二类“
            templateData.put("Veh_Dplb", "二类");
            Docx4jUtil.variableReplace(template, templateData);

            String previewPath = ResourceUtil.getConfigByName("previewPath") + pathSuffix;
            // 预览文件保存路径
            File docxFile = new File(previewPath);
            if (!docxFile.exists()) {
                docxFile.mkdirs();
            }
            String docxFileName = new StringBuilder("uniData_").append(certHgzPrintEO.getId()).append(".docx").toString();
            String docxFilePath = previewPath + docxFileName;
            Docx4jUtil.writeDocxToStream(template, docxFilePath);

            String pdfFileName = new StringBuilder("uniData_").append(certHgzPrintEO.getId()).append(".pdf").toString();
            String pdfFilePath = previewPath + pdfFileName;
            PDFUtils2.wordToPdf(new File(docxFilePath), new File(pdfFilePath), wordToPdfUrl);
            return pathSuffix + pdfFileName;
        } catch (FileNotFoundException e) {
            log.error(templateType.getName() + "打印模板不存在");
            throw new BusinessException(templateType.getName() + "打印模板不存在");
        } catch (BusinessException e) {
            throw e;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }


    @Override
    public CertificationEntity getCertificationEntityByCertificateDeclareEntity(
            CertHgzPrintEO c) {
        CertificationEntity cer = new CertificationEntity();
        cer.setId(c.getId());
        cer.setSerialNumber("");//流水号
        cer.setPageNumber(c.getVehZzbh());//纸张编号
        cer.setDiscernNumber(c.getVehClsbdh());//车辆识别代号
        cer.setCarNumber(c.getVehClxh());//车辆型号
        cer.setFrameNumber(c.getVehCjh());//车架号
        cer.setEngineNumber(c.getVehFdjh());//发动机号
        cer.setEngineModel(c.getVehFdjxh());//发动机型号
        cer.setCarColor(c.getVehCsys());//车颜色
        cer.setProducedDate(DateUtils.stringToDate("yyyy年MM月dd日", c.getVehClzzrq()));//车辆制造日期
        cer.setOpeningDate(DateUtils.stringToDate("yyyy年MM月dd日", c.getVehFzrq()));//发证日期
        cer.setVersionNumber(c.getHgzStatus());//版本号
        cer.setForwardReason(c.getHgzOverTimeReason());//补传原因
        cer.setLogoutReason(c.getHgzLogoutReason());//注销原因
        cer.setCarStyle(c.getHgzStyle());//车型款式
        cer.setCompanyID(c.getVehQyid());//企业id
        //todo 和生产单位不是一个？
        cer.setCompanyName(c.getVehClzzqymc());//企业名称
        cer.setNoticeProductNo(c.getVehCpggh());//公告产品号
        cer.setCarStateMsg(c.getHgzVehicleState());//车辆状态信息
        cer.setAnnouncementBatch(c.getHgzBatch());//公告批次
        cer.setAnnouncementDate(c.getHgzAnnounceEffectiveDate());//公告生效日期
        cer.setMakeCompanyName(c.getVehClzzqymc());//车辆制造企业名称
        cer.setCarType(c.getVehClfl());//车辆类型
        cer.setCarName(c.getVehClmc());//车辆名称
        cer.setCarBrand(c.getVehClpp());//车辆品牌
        cer.setAllocationNumber(c.getConfigSeqNo());//配置序列号
        cer.setChassisModel(c.getVehDpxh());//底盘型号
        cer.setChassisID(c.getVehDpid());//底盘id
        cer.setFuelKind(c.getVehRlzl());//燃料种类
        cer.setLetStandard(c.getVehPfbz());//排放标准
        cer.setDisplacement(c.getVehPl());//排量
        cer.setPower(c.getVehGl());//功率
        cer.setTurnType(c.getVehZxxs());//转向形式
        cer.setFrontGauge(c.getVehQlj());//前轮距
        cer.setTrackRear(c.getVehHlj());//后轮距
        cer.setTiresNo(c.getVehLts());//轮胎数
        cer.setTiresNorms(c.getVehLtgg());//轮胎规格
        cer.setSpringNo(c.getVehGbthps());//钢板弹簧片数
        cer.setWheelBase(c.getVehZj());//轴距
        cer.setAxleLoad(c.getVehZh());//轴荷
        cer.setAxleNo(c.getVehZs());//轴数
        cer.setTurnedWeeks(c.getVehZxzs());//转向轴数
        cer.setContourLong(c.getVehWkc());//外廓长
        cer.setContourWide(c.getVehWkc());//外廓宽
        cer.setContourHeight(c.getVehWkg());//外廓高
        cer.setBoxInsideLong(c.getVehHxnbc()
        );//货箱内部长
        cer.setBoxInsideWide(c.getVehHxnbg());//货箱内部宽
        cer.setBoxInsideHeight(c.getVehHxnbk());//货箱内部高
        cer.setSumWeight(c.getVehZzl());//总质量
        cer.setRatedWeight(c.getVehEdzzl());//额定载质量
        cer.setHostlingWeight(c.getVehZbzl());//整备质量
        cer.setWeightCoefficient(c.getVehZzllyxs());//载质量利用系数
        cer.setTractionWeight(c.getVehZqyzzl());//准牵引总质量
        cer.setRatedPassenger(c.getVehEdzk());//额定载客
        cer.setSaddleWeight(c.getVehBgcazzdyxzzl());//半挂车鞍座最大允许总质量
        cer.setPassengers(c.getVehJsszcrs());//驾驶室准乘人数
        //todo
        cer.setBeforeBrakingWay("");//前制动方式
        cer.setLaterBrakingWay("");//后制动方式
        cer.setBeforeBrakingHandle("");//前制动操作方式
        cer.setLaterBrakingHandle("");//后制动操作方式
        cer.setOilWear(c.getVehYh());//油耗
        cer.setMaximumSpeed(c.getVehZgcs());//最高车速
        cer.setRemark(c.getVehBz());//备注
        cer.setCompanyStandard(c.getVehQybz());//企业标准
        cer.setCarProductCompany(c.getVehClscdwmc());//车辆生产单位名称
        cer.setProductionAdd(c.getVehCpscdz());//产品生产地址
        cer.setCompanyElseMsg(c.getVehQyqtxx());//企业其他信息
        cer.setPureElectricSign(c.getVehCddbj());//纯电动标记
        cer.setTaxExemptionSign(c.getHgzMzsxnyqcml());//是否列入《免征车辆购置税的新能源汽车车型目录》
        cer.setReductionVehicleTax(c.getHgzJzclgzsgc());//是否属于减征车辆购置税挂车
        cer.setCerNumber(c.getVehZchgzbh());//整车合格证编号上传15位的完整合格证编号    20180908确认

        if (CertificateVehicleOrChassis.VEHICLE.getFlag() == c.getHgzType().intValue()) {//整车
            cer.setWzhgzbh(c.getVehZchgzbh());//整车完整合格证编号
            cer.setChassisNumber(c.getVehDphgzbh());//底盘合格证编号上传15位的完整合格证编号   20180908确认
        } else {//底盘
            cer.setWzhgzbh(c.getVehDphgzbh());//底盘完整合格证编号
            cer.setChassisNumber("");//底盘无底盘合格证编号
        }
        cer.setDywym(c.getVehDywym());//打印唯一码
        cer.setVeh_Jyw(c.getVehJyw());//校验码
        cer.setPrintTime(c.getHgzPrintTime());//打印完成时间
        cer.setUpdateReason("");//修改原因
        cer.setOptionDescribe("");//选装情况
        cer.setJfpzid("");//积分配置ID
        //TODO
        cer.setCreationTime(c.getCreateTime());//创建时间
        cer.setCreateDate(c.getCreateTime());//创建时间
			/*cer.setPrintStatus(serialNumber);//是否打印(1.未打印2.已打印3.修改未打印4.修改已打印5.撤销未打印6.撤销已打印)
			cer.setAuthorizationCode(serialNumber);//授权码
			cer.setUpdateName(serialNumber);//操作用户
			cer.setIssueReason(serialNumber);//转发原因
			cer.setModelNumber(serialNumber);//车辆识别代号,现在没有合同编号了
			cer.setState(serialNumber);//打印状态 ：0未打印 1已打印
			cer.setId(serialNumber);*/

        return cer;
    }

    //上传成功后进行移除操作
    @Override
    public void updateCerVersionUploadSuccess(CertHgzPrintEO certification, HttpServletRequest request) {
        if (CertificateVersionNumner.NO_UPLOAD.getValue().equals(certification.getHgzStatus())
                || CertificateVersionNumner.CANCLE_PRINT.getValue().equals(certification.getHgzStatus())
                || CertificateVersionNumner.CANCLE_PRINT.getValue().equals(certification.getHgzStatus())) {
            certification.setHgzStatus(CertificateVersionNumner.UPLOAD.getValue());

        } else {
            throw new AdcDaBaseException(certification.getHgzStatus() + "状态不允许上传");
        }

        certification.setHgzUploadTime(new Date());
        saveOrUpdate(certification);
    }

    /***
     * 合格证补传
     * @param
     * @param request
     * @return
     */
    @Override
    public String certificateSupplement(String id, String reason, HttpServletRequest request) {
        String msg = "";
        CertHgzPrintEO byId = getById(id);
        try {
            CertificateUploadBackMsg certBeanSupple = doCertificateSupplement(id, reason);
            if (certBeanSupple.isFlag()) {
                updateCerVersion(id, reason, request);
                msg = "合格证补传成功!";
            } else {
                msg = certBeanSupple.getMsg();
            }
        } catch (Exception e) {
            e.printStackTrace();
            msg = "车辆识别代号为" + byId.getVehClsbdh() + "合格证补传失败<br/>";
        }
        return msg;
    }

    //补传后修改版本号和菜单号
    @Override
    public void updateCerVersion(String id, String reason, HttpServletRequest request) {
        CertHgzPrintEO certification = getById(id);
        //保存补传原因
        certification.setHgzOverTimeReason(reason);
        //改变版本号
//        certification.setHgzStatus(CertificateVersionNumner.PRINT_UP.getValue());
//        certification.setMenuNumber(CertificationQueryStatus.UPLOAD_CERTIFICATE.getValue());
        certification.setHgzUploadTime(new Date());
        saveOrUpdate(certification);
    }

    public CertificateUploadBackMsg doCertificateSupplement(String id, String reason) {
        CertificateUploadBackMsg certBean = new CertificateUploadBackMsg();
        boolean flag = false;
        String msg = "";
        CertHgzPrintEO certification = getById(id);
        try {
            //整车数据进行补传
            CertificateInfo certificateInfo = new CertificateInfo();
            ArrayOfCertificateInfo cis = new ArrayOfCertificateInfo();
            cis.getCertificateInfo().add(certificateInfo);
            OperateResult message = CertificationUtils.uploadOverTimeEnt(cis, reason);

            if (message.getResultCode() == 1) {
                flag = true;
            }
        } catch (ExceptionInInitializerError e1) {
            e1.printStackTrace();
            msg = "调用接口失败，请检查是否插入U盾！";
        } catch (NoClassDefFoundError n) {
            n.printStackTrace();
            msg = "调用接口失败，请检查是否插入U盾！";
        } catch (Exception e) {
            e.printStackTrace();
            msg = "车辆识别代号为" + certification + "合格证补传失败<br/>";
        }
        certBean.setFlag(flag);
        certBean.setMsg(msg);
        return certBean;
    }

    //合格证修改成功后移除数据
    @Override
    public void toUpdateCerVersion(CertHgzPrintEO certificate, HttpServletRequest request) throws Exception {
        CertHgzPrintEO certificationOrcal = getById(certificate.getId());
        MyBeanUtils.copyBeanNotNull2Bean(certificate, certificationOrcal);
//        certificationOrcal.setHgzStatus(CertificateVersionNumner.UPDATE_UP.getValue());
//        certificationOrcal.setMenuNumber(CertificationQueryStatus.UPLOAD_CERTIFICATE.getValue());
        certificationOrcal.setHgzUploadTime(new Date());
        saveOrUpdate(certificationOrcal);
    }

    //合格证撤销成功后移除数据
    @Override
    public void logOutCerVersion(String id, String reason) {
        CertHgzPrintEO certificationOrcal = getById(id);
//        certificationOrcal.setHgzStatus(CertificateVersionNumner.CANCLE_NO_UP.getValue());
//        certificationOrcal.setMenuNumber(CertificationQueryStatus.CANCLE_CERTIFICATE_PRINT.getValue());
        certificationOrcal.setHgzLogoutReason(reason);
        if (certificationOrcal.getHgzRevokeNum() == 0) {
            certificationOrcal.setHgzRevokeNum(1);
        } else {
            certificationOrcal.setHgzRevokeNum(certificationOrcal.getHgzRevokeNum() + 1);
        }
        saveOrUpdate(certificationOrcal);
    }

    //撤销合格证
    @Override
    public String doLogout(String id, String reason) {
        String msg = "";
        CertHgzPrintEO byId = getById(id);
        try {
            CertificateUploadBackMsg certBeanCancle = logOut(id, reason);
            if (certBeanCancle.isFlag()) {
                logOutCerVersion(id, reason);
                msg = "合格证注销成功!";
            } else {
                msg = certBeanCancle.getMsg();
            }
        } catch (Exception e) {
            e.printStackTrace();
            msg = "车辆识别代号为" + byId.getVehClsbdh() + "合格证撤销失败<br/>";
        }
        return msg;
    }

    public CertificateUploadBackMsg logOut(String id, String reason) {
        CertificateUploadBackMsg certBean = new CertificateUploadBackMsg();
        boolean flag = false;
        String msg = "";
        OperateResult message = null;
        CertHgzPrintEO byId = getById(id);
        try {
            ArrayOfString wzhgzbhs = new ArrayOfString();
            wzhgzbhs.getString().add(byId.getVehZchgzbh());
            if (wzhgzbhs != null) {
                message = CertificationUtils.uploadDeleteEnt(wzhgzbhs, reason);
                if (message.getResultCode() == 1) {
                    flag = true;
                }
            }
        } catch (ExceptionInInitializerError e1) {
            e1.printStackTrace();
            msg = "调用接口失败，请检查是否插入U盾！";
        } catch (NoClassDefFoundError n) {
            n.printStackTrace();
            msg = "调用接口失败，请检查是否插入U盾！";
        } catch (Exception e) {
            e.printStackTrace();
            msg = "车辆识别代号为" + byId.getVehClsbdh() + "合格证撤销失败<br/>";
        }
        certBean.setFlag(flag);
        certBean.setMsg(msg);
        return certBean;
    }

    @Override
    public Boolean updateVehicleInfo(CertUploadVehicleInfo certUploadVehicleInfo) {
        CertHgzPrintEO certHgzPrintEO = getById(certUploadVehicleInfo.getId());
        if (null == certHgzPrintEO) {
            throw new AdcDaBaseException("该条信息不存在");
        }
        Date fzrq = com.adc.da.util.utils.DateUtils.stringToDate(certUploadVehicleInfo.getVehFzrq(), "yyyy-MM-dd");
        Date zzrq = com.adc.da.util.utils.DateUtils.stringToDate(certUploadVehicleInfo.getVehClzzrq(), "yyyy-MM-dd");
        ////修改状态为已修改待打印
        certHgzPrintEO.setHgzPrintStatus(CertificatePrintStateEnum.UPDATE_PRINT.getValue());
        certHgzPrintEO.setHgzStatus(CertificateVersionNumner.NO_PRINT.getValue());

        certHgzPrintEO.setVehZzbh(certUploadVehicleInfo.getVehZzbh());
        certHgzPrintEO.setVehClsbdh(certUploadVehicleInfo.getVehClsbdh());
        certHgzPrintEO.setVehDphgzbh(certUploadVehicleInfo.getVehDphgzbh());
        certHgzPrintEO.setVehFdjh(certUploadVehicleInfo.getVehFdjh());
        certHgzPrintEO.setVehCsys(certUploadVehicleInfo.getVehCsys());
        certHgzPrintEO.setVehZchgzbh(certUploadVehicleInfo.getVehZchgzbh());
        DateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日");
        certHgzPrintEO.setVehFzrq(sdf.format(fzrq));
        certHgzPrintEO.setVehClzzrq(sdf.format(zzrq));
        updateById(certHgzPrintEO);
        //修改单车信息
        QueryWrapper<VehicleCertEO> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(VehicleCertEO.VEHICLE_CERT_VIN, certUploadVehicleInfo.getVehClsbdh());
        VehicleCertEO vehicleCertEO = vehicleCertEOService.getOne(queryWrapper);
        vehicleCertEO.setPageNo(certUploadVehicleInfo.getVehZzbh());
        vehicleCertEO.setVin(certUploadVehicleInfo.getVehClsbdh());
        vehicleCertEO.setDphgzCertno(certUploadVehicleInfo.getVehDphgzbh());
        vehicleCertEO.setFdjNo(certUploadVehicleInfo.getVehFdjh());
        vehicleCertEO.setVehicleColor(certUploadVehicleInfo.getVehCsys());
        vehicleCertEO.setHbCertno(certUploadVehicleInfo.getVehZchgzbh());
        vehicleCertEO.setCertDate(fzrq);
        vehicleCertEO.setProduceDate(zzrq);
        return vehicleCertEOService.updateById(vehicleCertEO);
    }

    @Override
    public StatisticsInfo getStatistics(String qyId, String batch) {
        StatisticsInfo statisticsInfo = new StatisticsInfo();
        int declareVehicleType = this.baseMapper.getDeclareVehicleType(qyId, batch).size();
        int yzxPrintCount = this.baseMapper.countYzxPrint(qyId, batch);
        int hbqdPrintCount = this.baseMapper.countHbqdPrint(qyId, batch);
        int hgzPrintCount = this.lambdaQuery().eq(CertHgzPrintEO::getVehQyid, qyId).eq(CertHgzPrintEO::getVehGgpc, batch)
                .eq(CertHgzPrintEO::getHgzPrintStatus, CertificatePrintStateEnum.PRINT.getValue()).count();
        statisticsInfo.setDeclareVehicleType(declareVehicleType)
                .setYzxPrintCount(yzxPrintCount)
                .setHbqdPrintCount(hbqdPrintCount)
                .setHgzPrintCount(hgzPrintCount);
        return statisticsInfo;
    }

}
