package com.adc.da.vdaims.service.impl;

import com.adc.da.enums.CertDataEnum;
import com.adc.da.enums.ConfigSeqEnum;
import com.adc.da.util.exception.AdcDaBaseException;
import com.adc.da.util.utils.CollectionUtils;
import com.adc.da.util.utils.UserUtil;
import com.adc.da.vdaims.entity.*;
import com.adc.da.vdaims.dao.CertZkExtEODao;
import com.adc.da.vdaims.service.*;
import com.adc.da.vdaims.util.StringUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import java.io.Serializable;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 * 作者 gen
 * 日期 2021-08-25
 */
@Service
public class CertZkExtEOServiceImpl extends ServiceImpl<CertZkExtEODao, CertZkExtEO> implements ICertZkExtEOService {

    @Autowired
    IConfigSeqEOService configSeqEOService;

    @Autowired
    ICertZkEOService certZkEOService;

    @Autowired
    ICertConfigSeqTemplateEOService certConfigSeqTemplateEOService;

    @Override
    public CertZkExtEO getByConfigSeqId(Serializable configSeqId) {
        QueryWrapper<CertZkExtEO> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(CertZkExtEO.CERT_ZK_EXT_CONFIG_SEQ_ID, configSeqId);
        return getOne(queryWrapper);
    }

    @Override
    @Transactional
    public CertZkExtEO saveOrUpdate(CertZkExtEO entity, String configSeqId, boolean isSubmit) {
        ConfigSeqEO configSeqEO = configSeqEOService.getById(configSeqId);
        CertZkExtEO oldEntity = getByConfigSeqId(configSeqId);
        if (null != oldEntity && CertDataEnum.SUBMIT.getCode().equals(oldEntity.getStatus())) {
            throw new AdcDaBaseException("当前状态已提交,不可编辑");
        }
        QueryWrapper<CertZkExtEO> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq(CertZkExtEO.CERT_ZK_EXT_CONFIG_SEQ_ID, entity.getConfigSeqId());
        queryWrapper1.eq(CertZkExtEO.CERT_ZK_EXT_STYLE, entity.getStyle());
        if (StringUtil.isNotEmpty(entity.getId())) {
            queryWrapper1.ne(CertZkExtEO.CERT_ZK_EXT_ID, entity.getId());
        }
        if (CollectionUtils.isNotEmpty(list(queryWrapper1))) {
            throw new AdcDaBaseException("当前配置序列号该款型已存在,请重新输入");
        }

        if (isSubmit) {
            if (StringUtil.isEmpty(entity.getTemplateName())) {
                throw new AdcDaBaseException("模板名称不能为空!");
            }
            if (StringUtil.isEmpty(entity.getStyle())) {
                throw new AdcDaBaseException("款型不能为空!");
            }
            //提交
            entity.setStatus(CertDataEnum.SUBMIT.getCode());

            QueryWrapper<CertZkEO> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq(CertZkEO.CERT_ZK_CONFIG_SEQ_ID, configSeqId);
            CertZkEO certZkEO = certZkEOService.getOne(queryWrapper);
            if (null == certZkEO) {
                throw new AdcDaBaseException("请先进行信息确认并保存!");
            }
            //将序列号列表数据改为已完成
            configSeqEO.setStatus(ConfigSeqEnum.FINISH.getCode());
            //已完成后 向模板库中插入一条
            CertConfigSeqTemplateEO certConfigSeqTemplateEO = new CertConfigSeqTemplateEO();
            BeanUtils.copyProperties(configSeqEO, certConfigSeqTemplateEO);
            certConfigSeqTemplateEO.setConfigSeqId(configSeqId);
            certConfigSeqTemplateEO.setSource(0);
            certConfigSeqTemplateEO.setStyle(entity.getStyle());
            certConfigSeqTemplateEO.setTemplateName(entity.getTemplateName());
            Map<String, Object> currentUserInfo = UserUtil.getCurrentUserInfo();
            certConfigSeqTemplateEO.setCompanyName((String) currentUserInfo.get("NAME"));
            certConfigSeqTemplateEOService.saveOrUpdate(certConfigSeqTemplateEO);

        } else {
            //保存
            entity.setStatus(CertDataEnum.EDIT.getCode());
            configSeqEO.setStatus(ConfigSeqEnum.EDIT.getCode());
        }
        //别忘记同步修改款型
        configSeqEO.setStyle(entity.getStyle());
        configSeqEOService.updateById(configSeqEO);
        saveOrUpdate(entity);
        return entity;
    }
}
