package com.adc.da.vdaims.service.impl;

import com.adc.da.enums.CertificateVehicleOrChassis;
import com.adc.da.enums.ConfigSeqEnum;
import com.adc.da.util.exception.AdcDaBaseException;
import com.adc.da.util.utils.ClassUtils;
import com.adc.da.util.utils.CollectionUtils;
import com.adc.da.util.utils.StringUtils;
import com.adc.da.util.utils.UserUtil;
import com.adc.da.vdaims.dao.ConfigSeqEODao;
import com.adc.da.vdaims.entity.*;
import com.adc.da.vdaims.service.ICertZkEOService;
import com.adc.da.vdaims.service.IClcpZkEOService;
import com.adc.da.vdaims.service.IConfigSeqEOService;
import com.adc.da.vdaims.util.StringUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jdk.nashorn.internal.runtime.logging.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 * 作者 gen
 * 日期 2021-08-23
 */
@Service
@Logger
@Transactional
public class ConfigSeqEOServiceImpl extends ServiceImpl<ConfigSeqEODao, ConfigSeqEO> implements IConfigSeqEOService {
    private static final org.slf4j.Logger logger = LoggerFactory.getLogger(ConfigSeqEOServiceImpl.class);
    private static final List<String> csvTitleList = Arrays.asList(new String[]{"配置序列号", "车辆品牌", "车辆型号", "车辆名称", "配置描述"});
    private static Map<String, String> csv2PropertyMap;

    @Autowired
    private ICertZkEOService certZkEOService;

    @Autowired
    private CertZkExtEOServiceImpl certZkExtEOService;


    @Override
    public List<String> getCsvTitle() {
        return csvTitleList;
    }

    @Override
    public int importCsv(List<LinkedHashMap<String, Object>> dataList) {
        String currentUserId = UserUtil.getCurrentUserId();
        List<ConfigSeqEO> waitAddList = new ArrayList<>();
        //校验配置序列号
        List<String> configSeqNoList = new ArrayList<>();
        Map<String, Integer> configSeqNoCount = new HashMap<>();
        QueryWrapper<ConfigSeqEO> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("distinct  " + ConfigSeqEO.CERT_CONFIG_SEQ_CONFIG_SEQ_NO);
        List<ConfigSeqEO> configSeqEOList = list(queryWrapper);
        if (CollectionUtils.isNotEmpty(configSeqEOList)) {
            configSeqNoList = configSeqEOList.stream().map(configSeqEO -> configSeqEO.getConfigSeqNo()).collect(Collectors.toList());
        }
        for (int i = 0; i < dataList.size(); i++) {
            LinkedHashMap<String, Object> obj = dataList.get(i);
            Map<String, String> newObj = new HashMap<>();
            ConfigSeqEO configSeqEO = new ConfigSeqEO();
            for (int j = 0; j < obj.keySet().size(); j++) {
                String csv = obj.keySet().toArray()[j].toString();
                String key = getCsv2PropertyMap().get(csv);
                if (null != key) {
                    newObj.put(key, ((String) obj.get(csv)).trim());
                }
            }
            ClassUtils.setFieldValue(configSeqEO, newObj);
            configSeqEO.setStatus(ConfigSeqEnum.TOBEACK.getCode());
            configSeqEO.setAckStatus(ConfigSeqEnum.TOBEACK.getCode());
            String productModel = configSeqEO.getProductModel();
            if (StringUtil.isEmpty(configSeqEO.getConfigSeqNo())) {
                throw new AdcDaBaseException("配置序列号不能为空");
            }
            if (configSeqEO.getConfigSeqNo().trim().length() != 25) {
                throw new AdcDaBaseException("配置序列号应为25位");
            }

            if (StringUtils.isNotEmpty(productModel)) {
                //拆开品牌和型号
                int index = productModel.indexOf(configSeqEO.getBrand());
                if (index > -1) {
                    String model = productModel.substring(configSeqEO.getBrand().length(), productModel.length());
                    configSeqEO.setModel(model);
                } else {
                    throw new AdcDaBaseException("产品型号应包含品牌");
                }

            }
            if (configSeqEO.getProductName().contains("底盘")) {
                configSeqEO.setType(CertificateVehicleOrChassis.CHASSIS.getFlag());
            } else {
                configSeqEO.setType(CertificateVehicleOrChassis.VEHICLE.getFlag());
            }
            int beforeCount = configSeqNoCount.get(configSeqEO.getConfigSeqNo()) == null ? 0 : configSeqNoCount.get(configSeqEO.getConfigSeqNo());
            if (beforeCount == 1) {
                throw new AdcDaBaseException("配置序列号:" + configSeqEO.getConfigSeqNo() + "重复");
            } else {
                configSeqNoCount.put(configSeqEO.getConfigSeqNo(), 1);
            }
            if (configSeqNoList.contains(configSeqEO.getConfigSeqNo())) {
                throw new AdcDaBaseException("配置序列号:" + configSeqEO.getConfigSeqNo() + "已存在");
            }
            //添加车辆类别从配置描述中取出

            waitAddList.add(configSeqEO);
        }
        saveBatch(waitAddList);
        logger.info(Thread.currentThread() + "插入了数据" + waitAddList.size() + "条");
        return waitAddList.size();
    }

    @Override
    public boolean submit(Serializable... ids) {
        QueryWrapper<ConfigSeqEO> queryWrapper = new QueryWrapper<>();
        queryWrapper.in(ConfigSeqEO.CERT_CONFIG_SEQ_ID, ids);
        List<ConfigSeqEO> configSeqEOList = list(queryWrapper);
        for (ConfigSeqEO configSeqEO : configSeqEOList) {
            if (StringUtils.isEmpty(configSeqEO.getBatch())) {
                throw new AdcDaBaseException("序列号: " + configSeqEO.getConfigSeqNo() + "的批次不能为空");
            }
            if (StringUtils.isEmpty(configSeqEO.getVehicleType())) {
                throw new AdcDaBaseException("序列号: " + configSeqEO.getConfigSeqNo() + "的车辆类别不能为空");
            }
        }

        UpdateWrapper<ConfigSeqEO> updateWrapper = new UpdateWrapper();
        updateWrapper.in(ConfigSeqEO.CERT_CONFIG_SEQ_ID, ids);
        updateWrapper.eq(ConfigSeqEO.CERT_CONFIG_SEQ_STATUS, ConfigSeqEnum.TOBEACK.getCode());
        updateWrapper.set(ConfigSeqEO.CERT_CONFIG_SEQ_STATUS, ConfigSeqEnum.ACK.getCode());
        updateWrapper.set(ConfigSeqEO.CERT_CONFIG_SEQ_ACK_STATUS, ConfigSeqEnum.ACK.getCode());
        return update(updateWrapper);
    }

    @Override
    public boolean rollBack(Serializable... ids) {
        List<Serializable> idList = Arrays.asList(ids);
        QueryWrapper<ConfigSeqEO> queryWrapper = new QueryWrapper<>();
        queryWrapper.in(ConfigSeqEO.CERT_CONFIG_SEQ_ID, idList);
        List<ConfigSeqEO> configSeqEOList = list(queryWrapper);
        for (ConfigSeqEO configSeqEO : configSeqEOList) {
            if (ConfigSeqEnum.FINISH.getCode().equals(configSeqEO.getStatus())) {
                throw new AdcDaBaseException("存在已完成状态，不可回退");
            }
            configSeqEO.setStatus(ConfigSeqEnum.TOBEACK.getCode());
            configSeqEO.setAckStatus(ConfigSeqEnum.TOBEACK.getCode());
        }
        updateBatchById(configSeqEOList);

        QueryWrapper<CertZkEO> certZkEOQueryWrapper = new QueryWrapper<>();
        certZkEOQueryWrapper.in(CertZkEO.CERT_ZK_CONFIG_SEQ_ID, idList);
        certZkEOService.remove(certZkEOQueryWrapper);
        QueryWrapper<CertZkExtEO> certZkExtEOQueryWrapper = new QueryWrapper<>();
        certZkExtEOQueryWrapper.in(CertZkExtEO.CERT_ZK_EXT_CONFIG_SEQ_ID, idList);
        certZkExtEOService.remove(certZkExtEOQueryWrapper);
        return true;
    }

//    @Override
//    public Object getDetailById(Serializable id) {
//        ConfigSeqEO configSeqEO = getById(id);
//        if (null == configSeqEO) {
//            throw new AdcDaBaseException("暂无数据");
//        }
//        if (CertificateVehicleOrChassis.VEHICLE.getFlag() == configSeqEO.getType().intValue()) {
//            //整车
//            return certClcpZkEOService.getCertVehicleByConfigSeq(id);
//        } else {
//            //底盘
//        }
//        return null;
//    }

    public static Map<String, String> getCsv2PropertyMap() {
        if (null == csv2PropertyMap) {
            csv2PropertyMap = new LinkedHashMap<>();
            csv2PropertyMap.put("配置序列号", "configSeqNo");
            csv2PropertyMap.put("车辆品牌", "brand");
            csv2PropertyMap.put("车辆型号", "productModel");
            csv2PropertyMap.put("车辆名称", "productName");
            csv2PropertyMap.put("配置描述", "description");
        }
        return csv2PropertyMap;
    }
}
