package com.adc.da.vdaims.service.impl;

import com.adc.da.util.utils.CollectionUtils;
import com.adc.da.util.utils.StringUtils;
import com.adc.da.util.utils.UUID;
import com.adc.da.vdaims.entity.*;
import com.adc.da.vdaims.dao.SyxySerialNumberEODao;
import com.adc.da.vdaims.enums.PrintStatus;
import com.adc.da.vdaims.enums.SerialNumberType;
import com.adc.da.vdaims.exception.BusinessException;
import com.adc.da.vdaims.service.ISyxySerialNumberEOService;
import com.adc.da.vdaims.service.IVehicleCertEOService;
import com.adc.da.vdaims.service.IYzxBaseDataEOService;
import com.adc.da.vdaims.service.IYzxTemplateDataEOService;
import com.adc.da.vdaims.vo.SerialNumberVo;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Date;
import java.util.ArrayList;
import java.util.Collection;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicLong;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * <p>
 * 纸张编号管理 服务实现类
 * </p>
 * 作者 gen
 * 日期 2021-08-27
 */
@Service
public class SyxySerialNumberEOServiceImpl extends ServiceImpl<SyxySerialNumberEODao, SyxySerialNumberEO> implements ISyxySerialNumberEOService {

    /**
     * key:qyId-type  value:serialNumber
     */
    private static final Map<String, SerialNumberBean> serialNumberMap = new ConcurrentHashMap<>();

    private static Object lock = new Object();

    private static final Pattern pattern = Pattern.compile("[A-Za-z0-9]*[A-Za-z]+");

    @Autowired
    private IVehicleCertEOService vehicleCertEOService;

    @Autowired
    private IYzxTemplateDataEOService yzxTemplateDataEOService;

    @Override
    public void initSerialNumber() {
        serialNumberMap.clear();
        this.list().forEach(x -> {
            String key = x.getQyId() + "-" + x.getType();
            SerialNumberBean numberBean = new SerialNumberBean();
            numberBean.setPrefix(StringUtils.isBlank(x.getPrefix()) ? StringUtils.EMPTY : x.getPrefix())
            .setNumber(new AtomicLong(Long.valueOf(x.getSerialNumber()) - 1L))
            .setNumberLength(x.getSerialNumber().length());
            serialNumberMap.put(key, numberBean);
        });
    }

    @Override
    public String getNextSerialNumber(String qyId, SerialNumberType type) {
        synchronized (lock) {
            String key = qyId + "-" + type.getValue();
            SerialNumberBean numberBean = serialNumberMap.get(key);
            if (numberBean == null) {
                throw new BusinessException(type.getName() + "的纸张编号不存在：企业ID=" + qyId);
            }
            // 生成新的编号，防止异常时，编号保持着增加前的状态
            AtomicLong nextSerialNumber = new AtomicLong(numberBean.getNumber().get());
            String prefix = numberBean.getPrefix();
            String number = StringUtils.leftPad(String.valueOf(nextSerialNumber.incrementAndGet()), numberBean.getNumberLength(), "0");
            this.update(new UpdateWrapper<SyxySerialNumberEO>().set(SyxySerialNumberEO.SYXY_SERIAL_NUMBER_SERIAL_NUMBER, number)
                    .eq(SyxySerialNumberEO.SYXY_SERIAL_NUMBER_QY_ID, qyId).eq(SyxySerialNumberEO.SYXY_SERIAL_NUMBER_TYPE, type.getValue()));
            numberBean.setNumber(nextSerialNumber);
            return prefix + number;
        }
    }

    @Override
    public void updateSerialNumber(SerialNumberVo serialNumberVo) {
        SyxySerialNumberEO syxySerialNumberEO = this.getById(serialNumberVo.getId());
        if (syxySerialNumberEO == null) {
            throw new BusinessException("纸张编号数据不存在");
        }
        String oldKey = syxySerialNumberEO.getQyId() + "-" + syxySerialNumberEO.getType();
        String newKey = syxySerialNumberEO.getQyId() + "-" + serialNumberVo.getType();
        Matcher matcher = pattern.matcher(serialNumberVo.getPrefixSerialNumber());
        List<String> prefixList = new ArrayList<>();
        while (matcher.find()) {
            prefixList.add(matcher.group());
        }
        if (prefixList.size() > 1) {
            throw new BusinessException("编号格式错误");
        }
        String prefix = prefixList.isEmpty()? StringUtils.EMPTY : prefixList.get(0);
        String numberStr = serialNumberVo.getPrefixSerialNumber().substring(prefix.length());
        int numberLength = numberStr.length();
        AtomicLong number = new AtomicLong(Long.valueOf(numberStr) - 1L);
        synchronized (lock) {
            SerialNumberBean numberBean = new SerialNumberBean();
            numberBean.setPrefix(StringUtils.isBlank(prefix) ? StringUtils.EMPTY : prefix)
                    .setNumber(number)
                    .setNumberLength(numberLength);
            serialNumberMap.remove(oldKey);
            serialNumberMap.put(newKey, numberBean);
            syxySerialNumberEO.setSerialNumber(numberStr).setPrefix(prefix);
            this.updateById(syxySerialNumberEO);
            switch (SerialNumberType.getByValue(serialNumberVo.getType())) {
                case YZXZSDY:
                    // 初始化未打印和参考打印的一致性纸张编号
                    initYzxSerialNumber(syxySerialNumberEO.getQyId());
                    break;
                case HGZDY:
                    break;
                case HBQDDY:
                    break;
                default:
                    throw new BusinessException("纸张编号类型错误："+serialNumberVo.getType());
            }
        }
    }

    @Override
    public void addSerialNumber(String qyId, SerialNumberVo serialNumberVo) {
        Matcher matcher = pattern.matcher(serialNumberVo.getPrefixSerialNumber());
        List<String> prefixList = new ArrayList<>();
        while (matcher.find()) {
            prefixList.add(matcher.group());
        }
        if (prefixList.size() > 1) {
            throw new BusinessException("编号格式错误");
        }
        SerialNumberType serialNumberType = SerialNumberType.getByValue(serialNumberVo.getType());
        if (serialNumberType == SerialNumberType.NONE) {
            throw new BusinessException("编号类型错误");
        }
        Integer count = this.lambdaQuery().eq(SyxySerialNumberEO::getQyId, qyId).eq(SyxySerialNumberEO::getType, serialNumberType.getValue()).count();
        if (count != 0) {
            throw new BusinessException("存在相同类型的纸张编号");
        }
        String prefix = prefixList.isEmpty()? StringUtils.EMPTY : prefixList.get(0);
        String numberStr = serialNumberVo.getPrefixSerialNumber().substring(prefix.length());
        int numberLength = numberStr.length();
        AtomicLong number = new AtomicLong(Long.valueOf(numberStr) - 1L);
        SyxySerialNumberEO syxySerialNumberEO = new SyxySerialNumberEO();
        syxySerialNumberEO.setId(UUID.randomUUID())
                .setQyId(qyId)
                .setPrefix(prefix)
                .setSerialNumber(numberStr)
                .setType(serialNumberType.getValue())
                .setCreateDate(new Date());
        if (this.save(syxySerialNumberEO)) {
            SerialNumberBean numberBean = new SerialNumberBean();
            numberBean.setPrefix(StringUtils.isBlank(prefix) ? StringUtils.EMPTY : prefix)
                    .setNumber(number)
                    .setNumberLength(numberLength);
            String newKey = qyId + "-" + serialNumberType.getValue();
            serialNumberMap.put(newKey, numberBean);
            return;
        }
        throw new BusinessException("添加失败");
    }

    @Override
    public void delSerialNumber(String qyId, List<String> ids) {
        Collection<SyxySerialNumberEO> syxySerialNumberEOS = this.listByIds(ids);
        this.removeByIds(ids);
        syxySerialNumberEOS.forEach(x -> {
            String key = qyId + x.getType();
            serialNumberMap.remove(key);
        });
    }

    /**
     * 初始化未打印和参考打印的一致性纸张编号
     * @param qyId
     */
    private void initYzxSerialNumber(String qyId) {
        List<VehicleCertEO> vehicleCertEOList = vehicleCertEOService.lambdaQuery().eq(VehicleCertEO::getQyId, qyId).list();
        if (CollectionUtils.isEmpty(vehicleCertEOList)) {
            return;
        }
        List<String> idList = vehicleCertEOList.stream().filter(x-> StringUtils.isNoneBlank(x.getYzxCertno())).map(VehicleCertEO::getId).collect(Collectors.toList());
        List<YzxTemplateDataEO> yzxTemplateDataEOS = yzxTemplateDataEOService.lambdaQuery().in(YzxTemplateDataEO::getId, idList).ne(YzxTemplateDataEO::getPrintStatus, PrintStatus.PRINTED.getValue()).list();
        if (CollectionUtils.isEmpty(yzxTemplateDataEOS)) {
            return;
        }
        String key = qyId + "-" + SerialNumberType.YZXZSDY.getValue();
        SerialNumberBean numberBean = serialNumberMap.get(key);
        if (numberBean == null) {
            throw new BusinessException(SerialNumberType.YZXZSDY.getName() + "的纸张编号不存在：企业ID=" + qyId);
        }
        AtomicLong nextSerialNumber = new AtomicLong(numberBean.getNumber().get());
        yzxTemplateDataEOS.forEach(x->{
            x.setYzxPageNo(numberBean.getPrefix() + StringUtils.leftPad(String.valueOf(nextSerialNumber.incrementAndGet()), numberBean.getNumberLength(),"0"));
        });
        yzxTemplateDataEOService.updateBatchById(yzxTemplateDataEOS);
        String number = StringUtils.leftPad(String.valueOf(nextSerialNumber.get() + 1L), numberBean.getNumberLength(), "0");
        this.update(new UpdateWrapper<SyxySerialNumberEO>().set(SyxySerialNumberEO.SYXY_SERIAL_NUMBER_SERIAL_NUMBER, number)
                .eq(SyxySerialNumberEO.SYXY_SERIAL_NUMBER_QY_ID, qyId).eq(SyxySerialNumberEO.SYXY_SERIAL_NUMBER_TYPE, SerialNumberType.YZXZSDY.getValue()));
        numberBean.setNumber(nextSerialNumber);
    }


}
