package com.adc.da.vdaims.service.impl;

import com.adc.da.enums.CertificatePrintStateEnum;
import com.adc.da.enums.CertificateVehicleOrChassis;
import com.adc.da.enums.CertificateVersionNumner;
import com.adc.da.util.exception.AdcDaBaseException;
import com.adc.da.util.utils.CollectionUtils;
import com.adc.da.util.utils.DateUtils;
import com.adc.da.util.utils.UserUtil;
import com.adc.da.vdaims.dao.VehicleCertEODao;
import com.adc.da.vdaims.entity.CertHgzPrintEO;
import com.adc.da.vdaims.entity.VehicleCertEO;
import com.adc.da.vdaims.entity.VehicleCertInfoEO;
import com.adc.da.vdaims.service.IVehicleCertEOService;
import com.adc.da.vdaims.util.StringUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * <p>
 * 服务实现类
 * </p>
 * 作者 gen
 * 日期 2021-08-24
 */
@Service
public class VehicleCertEOServiceImpl extends ServiceImpl<VehicleCertEODao, VehicleCertEO> implements IVehicleCertEOService {

    @Autowired
    CertHgzPrintEOServiceImpl certHgzPrintEOService;

    @Autowired
    VehicleCertInfoEOServiceImpl vehicleCertInfoEOService;

    @Autowired
    CertZkEOServiceImpl certZkEOService;


    public VehicleCertEO getVehicleCertByExcel(Row key, Row value) throws ParseException {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        int column = key.getPhysicalNumberOfCells();
        VehicleCertEO vehicleCertEO = new VehicleCertEO();
        for (int c = 0; c < column; c++) {
            String s_key = key.getCell(c) == null ? "" : key.getCell(c).toString().trim();
            String s_value = "";
            if (null != value.getCell(c)) {

                if (value.getCell(c).getCellType() == Cell.CELL_TYPE_STRING) {
                    s_value = value.getCell(c) == null ? "" : value.getCell(c).getStringCellValue().trim();
                } else if (value.getCell(c).getCellType() == Cell.CELL_TYPE_NUMERIC) {
                    s_value = value.getCell(c) == null ? "" : String.valueOf(value.getCell(c).getNumericCellValue()).trim();
                } else {
                    s_value = value.getCell(c) == null ? "" : value.getCell(c).toString().trim();
                }
            }
            Date date;
            switch (s_key) {
                case "配置序列号":
                    if (null == s_value || StringUtil.isEmpty(s_value)) {
                        throw new AdcDaBaseException("配置序列号不能为空");
                    }
                    if (s_value.length() != 25) {
                        throw new AdcDaBaseException("配置序列号长度应为25位");
                    }
                    vehicleCertEO.setConfigSeqNo(s_value);
                    break;
                case "款型":
                    if (null == s_value || StringUtil.isEmpty(s_value)) {
                        throw new AdcDaBaseException("款型不能为空");
                    }
                    vehicleCertEO.setVehicleStyle(s_value);
                    break;
                case "VIN编号":
                    if (null == s_value || StringUtil.isEmpty(s_value)) {
                        throw new AdcDaBaseException("VIN编号不能为空");
                    }
                    vehicleCertEO.setVin(s_value);
                    break;
                case "发动机编号":
                    if (null == s_value || StringUtil.isEmpty(s_value)) {
                        throw new AdcDaBaseException("发动机编号不能为空");
                    }
                    vehicleCertEO.setFdjNo(s_value);
                    break;
                case "驱动电机编号(包括型号和序列号,按驱动电机编号实际打刻形式填写)":
                    vehicleCertEO.setQddjNo(s_value);
                    break;
                case "车身颜色":
                    if (null == s_value || StringUtil.isEmpty(s_value)) {
                        throw new AdcDaBaseException("车身颜色不能为空");
                    }
                    vehicleCertEO.setVehicleColor(s_value);
                    break;
                case "纸张编号":
                    if (null == s_value || StringUtil.isEmpty(s_value)) {
                        throw new AdcDaBaseException("纸张编号不能为空");
                    }
                    vehicleCertEO.setPageNo(s_value);
                    break;
                case "合格证证书编号":
                    vehicleCertEO.setHgzCertno(s_value);
                    break;
                case "底盘合格证编号":
                    vehicleCertEO.setDphgzCertno(s_value);
                    break;
                case "发证日期（如若不填默认为当前时间）":
                    date = StringUtil.isEmpty(s_value) ? new Date() : dateFormat.parse(s_value);
                    vehicleCertEO.setCertDate(date);
                    break;
                case "制造日期":
                    if (StringUtil.isEmpty(s_value) || StringUtil.isEmpty(s_value)) {
                        throw new AdcDaBaseException("制造日期不能为空");
                    }
                    date = StringUtil.isEmpty(s_value) ? new Date() : dateFormat.parse(s_value);
                    vehicleCertEO.setProduceDate(date);
                    break;
                case "一致性证书编号":
                    vehicleCertEO.setYzxCertno(s_value);
                    break;
                case "环保清单编号":
                    vehicleCertEO.setHbCertno(s_value);
                    break;
                default:
                    break;
            }
        }
        return vehicleCertEO;
    }

    @Override
    @Transactional
    public boolean submit(Serializable... ids) {

        List<Serializable> idList = Arrays.asList(ids);
        QueryWrapper<VehicleCertEO> queryWrapper = new QueryWrapper<>();
        queryWrapper.in(VehicleCertEO.VEHICLE_CERT_ID, idList);
        queryWrapper.eq(VehicleCertEO.VEHICLE_CERT_HGZ_STATUS, 1);
        if (CollectionUtils.isNotEmpty(list(queryWrapper))) {
            throw new AdcDaBaseException("存在已确认数据");
        }

        UpdateWrapper<VehicleCertEO> updateWrapper = new UpdateWrapper();
        updateWrapper.in(VehicleCertEO.VEHICLE_CERT_ID, idList);
        updateWrapper.eq(VehicleCertEO.VEHICLE_CERT_HGZ_STATUS, 0);
        updateWrapper.set(VehicleCertEO.VEHICLE_CERT_HGZ_STATUS, 1);
        Date ackDate = new Date();
        updateWrapper.set(VehicleCertEO.VEHICLE_CERT_HGZ_ACK_TIME, ackDate);//已确认的数据添加到待打印表里
        List<CertHgzPrintEO> hgzPrintByVehicleCert = null;
        try {
            hgzPrintByVehicleCert = getHgzPrintByVehicleCert(idList);
        } catch (ParseException e) {
            throw new AdcDaBaseException("时间日期格式错误");
        }

        for (CertHgzPrintEO certHgzPrintEO : hgzPrintByVehicleCert) {
            certHgzPrintEO.setHgzAckTime(ackDate);
        }
        certHgzPrintEOService.saveBatch(hgzPrintByVehicleCert);
        return update(updateWrapper);
    }


    @Override
    @Transactional
    public boolean returnHgz(Serializable... vins) {

        List<Serializable> vinList = Arrays.asList(vins);
        QueryWrapper<CertHgzPrintEO> queryWrapper = new QueryWrapper<>();
        queryWrapper.in(CertHgzPrintEO.CERT_HGZ_PRINT_VEH_CLSBDH, vinList);
        queryWrapper.eq(CertHgzPrintEO.CERT_HGZ_PRINT_HGZ_PRINT_STATUS, 2);
        if (CollectionUtils.isNotEmpty(certHgzPrintEOService.list(queryWrapper))) {
            throw new AdcDaBaseException("存在已打印数据");
        }

        QueryWrapper<CertHgzPrintEO> removeWrapper = new QueryWrapper<>();
        removeWrapper.in(CertHgzPrintEO.CERT_HGZ_PRINT_VEH_CLSBDH, vinList);
//        removeWrapper.ne(CertHgzPrintEO.CERT_HGZ_PRINT_HGZ_PRINT_STATUS, 2);
        certHgzPrintEOService.remove(removeWrapper);


        UpdateWrapper<VehicleCertEO> updateWrapper = new UpdateWrapper();
        updateWrapper.in(VehicleCertEO.VEHICLE_CERT_VIN, vinList);
        updateWrapper.eq(VehicleCertEO.VEHICLE_CERT_HGZ_STATUS, 1);
        updateWrapper.set(VehicleCertEO.VEHICLE_CERT_HGZ_STATUS, 0);
//        Date ackDate = new Date();
//        updateWrapper.set(VehicleCertEO.VEHICLE_CERT_HGZ_ACK_TIME, ackDate);//已确认的数据添加到待打印表里
        return update(updateWrapper);
    }

    List<CertHgzPrintEO> getHgzPrintByVehicleCert(List vehicleCertIds) throws ParseException {
        QueryWrapper<VehicleCertInfoEO> vehicleCertInfoEOQueryWrapper = new QueryWrapper<>();
        vehicleCertInfoEOQueryWrapper.in(VehicleCertInfoEO.V_VEHICLE_CERT_INFO_ID, vehicleCertIds);
        List<VehicleCertInfoEO> vehicleInfoList = vehicleCertInfoEOService.list(vehicleCertInfoEOQueryWrapper);
        List<CertHgzPrintEO> certHgzPrintEOList = new ArrayList<>();
        CertHgzPrintEO certHgzPrintEO = null;
        for (VehicleCertInfoEO vehicleCertInfoEO : vehicleInfoList) {

            if (vehicleCertInfoEO.getType().equals(CertificateVehicleOrChassis.VEHICLE.getFlag())) {
                //整车
                //1 车辆状态信息 取值为 QX 和 DP
                certHgzPrintEO = createVehicleCertificateBean(vehicleCertInfoEO);
            } else {
                //底盘
                certHgzPrintEO = createChassisBean(vehicleCertInfoEO);
            }
            certHgzPrintEO.setConfigSeqNo(vehicleCertInfoEO.getConfigSeqNo());
            certHgzPrintEO.setHgzAckTime(vehicleCertInfoEO.getHgzAckTime());
            certHgzPrintEO.setHgzBatch(vehicleCertInfoEO.getBatch());
            certHgzPrintEO.setHgzType(vehicleCertInfoEO.getType());
            certHgzPrintEO.setHgzRevokeNum(0);
            certHgzPrintEO.setHgzUpdateNum(0);
            certHgzPrintEO.setHgzPrintStatus(CertificatePrintStateEnum.NO_PRINT.getValue());
            certHgzPrintEO.setHgzStatus(CertificateVersionNumner.NO_PRINT.getValue());
            certHgzPrintEOList.add(certHgzPrintEO);
        }
        return certHgzPrintEOList;
    }


    public VehicleCertEO modify(VehicleCertEO entity) {
        QueryWrapper<VehicleCertEO> queryWrapper = new QueryWrapper();
        queryWrapper.eq(VehicleCertEO.VEHICLE_CERT_VIN, entity.getVin());
        queryWrapper.eq(VehicleCertEO.VEHICLE_CERT_CONFIG_SEQ_NO, entity.getConfigSeqNo());
        List<VehicleCertEO> list = list(queryWrapper);
        entity.setQyId(UserUtil.getCurrentUserName());
        if (CollectionUtils.isNotEmpty(list)) {
            VehicleCertEO old = list.get(0);
            entity.setId(old.getId());
            checkUpdate(entity);
        } else {
            checkSave(entity);
        }
        saveOrUpdate(entity);
        return entity;
    }


    public boolean removeByVin(Serializable... vinList) {
        QueryWrapper<VehicleCertEO> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.in(VehicleCertEO.VEHICLE_CERT_VIN, vinList);
        remove(queryWrapper1);
        QueryWrapper<CertHgzPrintEO> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.in(CertHgzPrintEO.CERT_HGZ_PRINT_VEH_CLSBDH, vinList);
        return certHgzPrintEOService.remove(queryWrapper2);
    }

    @Override
    public boolean updateById(VehicleCertEO entity) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq(VehicleCertEO.VEHICLE_CERT_VIN, entity.getVin());
//        queryWrapper.eq(VehicleCertEO.VEHICLE_CERT_QY_ID, UserUtil.getCurrentUserName());
        queryWrapper.ne(VehicleCertEO.VEHICLE_CERT_ID, entity.getId());
        if (CollectionUtils.isNotEmpty(list(queryWrapper))) {
            throw new AdcDaBaseException("VIN已存在，请重新输入");
        }
        entity.setQyId(UserUtil.getCurrentUserName());
        return super.updateById(entity);
    }

    /***
     * 构造整车打印信息
     *
     */
    public CertHgzPrintEO createVehicleCertificateBean(VehicleCertInfoEO vehicleCertInfoEO) throws ParseException {
        CertHgzPrintEO certHgzPrintEO = new CertHgzPrintEO();
        //2 整车合格证编号 4位企业代码+10位顺序号成功调用打印方法后可以通过该属性获得15位的整车合格证编号
        certHgzPrintEO.setVehZchgzbh(vehicleCertInfoEO.getHgzCertno());
        //3 底盘合格证编号
        certHgzPrintEO.setVehDphgzbh(vehicleCertInfoEO.getDphgzCertno());
        //4 发证日期
        certHgzPrintEO.setVehFzrq(DateUtils.dateToString(vehicleCertInfoEO.getCertDate(), "yyyy年MM月dd日"));
        //5 车辆制造企业名称
        certHgzPrintEO.setVehClzzqymc(vehicleCertInfoEO.getCompanyName());
        //6 企业 ID
        certHgzPrintEO.setVehQyid(vehicleCertInfoEO.getQyId());
        //7 车辆分类
        certHgzPrintEO.setVehClfl(vehicleCertInfoEO.getVehicleType());
        //8 车辆名称
        certHgzPrintEO.setVehClmc(vehicleCertInfoEO.getProductName());
        //9 车辆品牌
        certHgzPrintEO.setVehClpp(vehicleCertInfoEO.getBrand());
        //10 车辆型号
        certHgzPrintEO.setVehClxh(vehicleCertInfoEO.getModel());
        //11
        certHgzPrintEO.setVehCsys(vehicleCertInfoEO.getVehicleColor());
        //12
        certHgzPrintEO.setVehDpxh(vehicleCertInfoEO.getDpModel());
        //13
        certHgzPrintEO.setVehDpid(vehicleCertInfoEO.getDpid());
        //14
        certHgzPrintEO.setVehClsbdh(vehicleCertInfoEO.getVin());
        //15 车架号不能填
        certHgzPrintEO.setVehCjh(null);
        //16
        certHgzPrintEO.setVehFdjh(vehicleCertInfoEO.getFdjNo());
        //17
        certHgzPrintEO.setVehFdjxh(vehicleCertInfoEO.getMfadongji());
        //18
        certHgzPrintEO.setVehRlzl(vehicleCertInfoEO.getFuelType());
        //19
        certHgzPrintEO.setVehPfbz(vehicleCertInfoEO.getStandard());
        //20
        certHgzPrintEO.setVehPl(vehicleCertInfoEO.getMpailiang());
        //21
        certHgzPrintEO.setVehGl(vehicleCertInfoEO.getMgonglv());
        //22
        certHgzPrintEO.setVehZxxs(vehicleCertInfoEO.getZhuanxiangXingshi());
        //23
        certHgzPrintEO.setVehQlj(vehicleCertInfoEO.getQianlunju());
        //24
        certHgzPrintEO.setVehHlj(vehicleCertInfoEO.getHoulunju());
        //25
        certHgzPrintEO.setVehLts(vehicleCertInfoEO.getLuntaiNum());
        //26
        certHgzPrintEO.setVehLtgg(vehicleCertInfoEO.getLuntaiGuige());
        //27
        certHgzPrintEO.setVehGbthps(vehicleCertInfoEO.getTanhuangpianNum());
        //28
        certHgzPrintEO.setVehZj(vehicleCertInfoEO.getZhouju());
        //29
        certHgzPrintEO.setVehZh(vehicleCertInfoEO.getZhouhe());
        //30
        certHgzPrintEO.setVehZs(vehicleCertInfoEO.getZhouNum());
        //31
        certHgzPrintEO.setVehWkc(vehicleCertInfoEO.getLength());
        //32
        certHgzPrintEO.setVehWkk(vehicleCertInfoEO.getWidth());
        //33
        certHgzPrintEO.setVehWkg(vehicleCertInfoEO.getHeight());
        //34不填
        certHgzPrintEO.setVehHxnbc(vehicleCertInfoEO.getHuoxiangLength());
        //35
        certHgzPrintEO.setVehHxnbk(vehicleCertInfoEO.getHuoxiangWidth());
        //36
        certHgzPrintEO.setVehHxnbg(vehicleCertInfoEO.getHuoxiangHeight());
        //37
        certHgzPrintEO.setVehZzl(vehicleCertInfoEO.getTotalWeight());
        //38额定在质量不用填
        certHgzPrintEO.setVehEdzzl(vehicleCertInfoEO.getEdingQuality());
        //39
        certHgzPrintEO.setVehZbzl(vehicleCertInfoEO.getZhengbeiQuality());
        //40
        certHgzPrintEO.setVehZzllyxs(vehicleCertInfoEO.getZaiQuality());
        //41 todo
        certHgzPrintEO.setVehZqyzzl(vehicleCertInfoEO.getZhengbeiQuality());
        //42
        certHgzPrintEO.setVehEdzk(vehicleCertInfoEO.getEdingzaike());
        //43
        certHgzPrintEO.setVehBgcazzdyxzzl(vehicleCertInfoEO.getBanguaanzuo());
        //44
        certHgzPrintEO.setVehJsszcrs(vehicleCertInfoEO.getQianpaiChengke());
        //45
        certHgzPrintEO.setVehZgcs(vehicleCertInfoEO.getMaxSpeed());
        //46
        certHgzPrintEO.setVehClzzrq(DateUtils.dateToString(vehicleCertInfoEO.getProduceDate(), "yyyy年MM月dd日"));
        String remark = "-";
        if (StringUtil.isNotEmpty(vehicleCertInfoEO.getRemark())) {
            Pattern CRLF = Pattern.compile("(\r\n|\r|\n|\n\r)");
            Matcher m = CRLF.matcher(vehicleCertInfoEO.getRemark());
            if (m.find()) {
                remark = m.replaceAll("%%");
            }
        }

        //47
        certHgzPrintEO.setVehBz(remark);//备注 替换回车换行
        //48
        certHgzPrintEO.setVehQybz(vehicleCertInfoEO.getEnterpriseStandard());
        //49
        certHgzPrintEO.setVehClscdwmc(vehicleCertInfoEO.getEnterpriseName());
        //50
        certHgzPrintEO.setVehCpscdz(vehicleCertInfoEO.getProduceAddr());
        //51
        certHgzPrintEO.setVehQyqtxx(vehicleCertInfoEO.getEnterpriseOther());
        //52
        certHgzPrintEO.setVehZxzs(vehicleCertInfoEO.getZhuanxiangzhouNum());
        //53
        certHgzPrintEO.setVehTmxx("");
        //54
        certHgzPrintEO.setVehJyw("");
        //55
        certHgzPrintEO.setVehYh(vehicleCertInfoEO.getYouhao());
        if (null != vehicleCertInfoEO.getPureElectricity()) {
            //56
            certHgzPrintEO.setVehCddbj(String.valueOf(vehicleCertInfoEO.getPureElectricity() == 0 ? 2 : 1));
        }

        //57-64
        //65
        //66
        certHgzPrintEO.setVehCpggh(vehicleCertInfoEO.getProductNo() + vehicleCertInfoEO.getConfigSeqNo());
        //67
        certHgzPrintEO.setVehZzbh(vehicleCertInfoEO.getPageNo());
        //68
        certHgzPrintEO.setVehGgpc(vehicleCertInfoEO.getBatch());
        //69
        SimpleDateFormat sdf2 = new SimpleDateFormat("yyyy-MM-dd");
        Date effectiveDate = DateUtils.parseDate(vehicleCertInfoEO.getEffectiveDate(), "yyyyMMdd");
        certHgzPrintEO.setVehGgsxrq(sdf2.format(effectiveDate));

        certHgzPrintEO.setVehClztxx("QX");
        return certHgzPrintEO;
    }

    /***
     * 构造底盘打印信息

     */
    public CertHgzPrintEO createChassisBean(VehicleCertInfoEO vehicleCertInfoEO) throws ParseException {
        CertHgzPrintEO certHgzPrintEO = new CertHgzPrintEO();
        certHgzPrintEO.setVehClztxx("DP");//车辆状态信息
        certHgzPrintEO.setVehFzrq(DateUtils.dateToString(vehicleCertInfoEO.getCertDate(), "yyyy年MM月dd日")); //发证日期
        certHgzPrintEO.setVehClzzqymc(vehicleCertInfoEO.getCompanyName());//车辆制造企业名称
        certHgzPrintEO.setVehClfl(vehicleCertInfoEO.getVehicleType()); //车辆 类型
        certHgzPrintEO.setVehClmc(vehicleCertInfoEO.getProductName()); //车辆名称
        certHgzPrintEO.setVehClpp(vehicleCertInfoEO.getBrand());//车辆品牌
        certHgzPrintEO.setVehClxh(vehicleCertInfoEO.getModel()); //车辆型号
        certHgzPrintEO.setVehCsys(vehicleCertInfoEO.getVehicleColor()); //车辆颜色
        certHgzPrintEO.setVehDpid(vehicleCertInfoEO.getDpid()); //底盘ID
        certHgzPrintEO.setVehClsbdh(vehicleCertInfoEO.getVin());//车辆识别代号
        certHgzPrintEO.setVehCjh(vehicleCertInfoEO.getVin());//车架号
        certHgzPrintEO.setVehFdjh(vehicleCertInfoEO.getFdjNo()); //发动机号
        certHgzPrintEO.setVehFdjxh(vehicleCertInfoEO.getMfadongji());//发动机型号
        certHgzPrintEO.setVehRlzl(vehicleCertInfoEO.getFuelType()); //燃料种类
        certHgzPrintEO.setVehPfbz(vehicleCertInfoEO.getStandard()); //排放标准
        certHgzPrintEO.setVehPl(vehicleCertInfoEO.getMpailiang()); //排量
        certHgzPrintEO.setVehGl(vehicleCertInfoEO.getMgonglv());//功率
        certHgzPrintEO.setVehZxxs(vehicleCertInfoEO.getZhuanxiangXingshi());//转向形式
        certHgzPrintEO.setVehQlj(vehicleCertInfoEO.getQianlunju()); //前轮距
        certHgzPrintEO.setVehHlj(vehicleCertInfoEO.getHoulunju()); //后轮距
        certHgzPrintEO.setVehLts(vehicleCertInfoEO.getLuntaiNum());//轮胎数
        certHgzPrintEO.setVehLtgg(vehicleCertInfoEO.getLuntaiGuige());//轮胎规格
        certHgzPrintEO.setVehGbthps(vehicleCertInfoEO.getTanhuangpianNum()); //钢板弹簧片数
        certHgzPrintEO.setVehZj(vehicleCertInfoEO.getZhouju());//轴距
        certHgzPrintEO.setVehZh(vehicleCertInfoEO.getZhouhe());//轴荷
        certHgzPrintEO.setVehZs(vehicleCertInfoEO.getZhouNum());//轴数
        certHgzPrintEO.setVehWkc(vehicleCertInfoEO.getLength());//外廓长
        certHgzPrintEO.setVehWkk(vehicleCertInfoEO.getWidth());//外廓宽
        certHgzPrintEO.setVehWkg(vehicleCertInfoEO.getHeight());//外廓高
        certHgzPrintEO.setVehZzl(vehicleCertInfoEO.getTotalWeight());//总质量
        certHgzPrintEO.setVehZbzl(vehicleCertInfoEO.getZhengbeiQuality());//整备质量
        certHgzPrintEO.setVehZqyzzl(vehicleCertInfoEO.getZhengbeiQuality());//准牵引总质量
        certHgzPrintEO.setVehBgcazzdyxzzl(vehicleCertInfoEO.getBanguaanzuo());//半挂车最大允许质量
        certHgzPrintEO.setVehJsszcrs(vehicleCertInfoEO.getQianpaiChengke());//驾驶室准成人数
        certHgzPrintEO.setVehZgcs(vehicleCertInfoEO.getMaxSpeed());//最高车速
        String remark = "-";
        if (StringUtil.isNotEmpty(vehicleCertInfoEO.getRemark())) {
            Pattern CRLF = Pattern.compile("(\r\n|\r|\n|\n\r)");
            Matcher m = CRLF.matcher(vehicleCertInfoEO.getRemark());
            if (m.find()) {
                remark = m.replaceAll("%%");
            }
        }
        certHgzPrintEO.setVehBz(remark);//备注 替换回车换行
        certHgzPrintEO.setVehQybz(vehicleCertInfoEO.getEnterpriseName());//企业标准
        certHgzPrintEO.setVehClscdwmc(vehicleCertInfoEO.getEnterpriseName()); //车辆生产单位名称
        certHgzPrintEO.setVehCpscdz(vehicleCertInfoEO.getProduceAddr());//车辆生产单位地址
        certHgzPrintEO.setVehQyqtxx(vehicleCertInfoEO.getOther());//企业其他信息
        certHgzPrintEO.setVehClzzrq(DateUtils.dateToString(vehicleCertInfoEO.getProduceDate(), "yyyy年MM月dd日")); //车辆制造日期
        certHgzPrintEO.setVehCpggh(vehicleCertInfoEO.getProductNo() + vehicleCertInfoEO.getConfigSeqNo());//产品公告号
        certHgzPrintEO.setVehGgpc(String.valueOf(vehicleCertInfoEO.getBatch()));//公告批次
        SimpleDateFormat sdf2 = new SimpleDateFormat("yyyy-MM-dd");
        Date effectiveDate = DateUtils.parseDate(vehicleCertInfoEO.getEffectiveDate(), "yyyyMMdd");
        certHgzPrintEO.setVehGgsxrq(sdf2.format(effectiveDate));
        certHgzPrintEO.setVehDphgzbh(vehicleCertInfoEO.getDphgzCertno());//底盘合格证编号
        certHgzPrintEO.setVehZxzs(vehicleCertInfoEO.getZhuanxiangzhouNum());//转向轴个数
        //油耗
        //底盘型号
        //底盘合格证编号
        //额定载质量
        //载质量利用系数
        //额定载客（人）
        //车辆识别代号，为了申报重新打印时可以更新所以添加
        return certHgzPrintEO;

    }


}
