package com.adc.da.vdaims.service.impl;

import cn.hutool.json.JSONUtil;
import com.adc.da.util.utils.*;
import com.adc.da.util.utils.UUID;
import com.adc.da.vdaims.consts.CommConstant;
import com.adc.da.vdaims.dao.CertHgzPrintEODao;
import com.adc.da.vdaims.entity.*;
import com.adc.da.vdaims.dao.YzxBaseDataEODao;
import com.adc.da.vdaims.enums.ConfirmStatus;
import com.adc.da.vdaims.enums.PrintStatus;
import com.adc.da.vdaims.enums.SerialNumberType;
import com.adc.da.vdaims.enums.TemplateType;
import com.adc.da.vdaims.enums.DeclareStatus;
import com.adc.da.vdaims.exception.BusinessException;
import com.adc.da.vdaims.service.*;
import com.adc.da.vdaims.thread.TheadFactoryName;
import com.adc.da.vdaims.util.Docx4jUtil;
import com.adc.da.vdaims.util.GoogleQRCodeUtil;
import com.adc.da.vdaims.util.PDFUtils2;
import com.adc.da.vdaims.util.ResourceUtil;
import com.adc.da.vdaims.vo.*;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.apache.catalina.connector.Request;
import org.apache.catalina.connector.RequestFacade;
import org.apache.catalina.core.StandardContext;
import org.apache.catalina.session.StandardManager;
import org.apache.catalina.session.StandardSession;
import org.apache.http.HttpEntity;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.io.*;
import java.util.*;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 * 作者 gen
 * 日期 2021-08-25
 */
@Slf4j
@Service
public class YzxBaseDataEOServiceImpl extends ServiceImpl<YzxBaseDataEODao, YzxBaseDataEO> implements IYzxBaseDataEOService {
    @Autowired
    private IVehicleCertEOService vehicleCertEOService;
    @Autowired
    private IYzxTemplateDataEOService yzxTemplateDataEOService;
    @Autowired
    private ISyxySerialNumberEOService syxySerialNumberEOService;
    @Autowired
    private CertHgzPrintEODao certHgzPrintEODao;
    @Value("${qr.batchEncryptUrl}")
    private String batchEncryptUrl;
    @Value("${qr.wordToPdfUrl}")
    private String wordToPdfUrl;

    @Override
    public IPage<VehicleCertYzxVo> selectYzxInfoList(Page<VehicleCertYzxVo> page, Wrapper<VehicleCertYzxVo> queryWrapper, String qyId) {
        return this.baseMapper.selectYzxPageVo(page, queryWrapper, qyId);
    }

    /**
     * 同步官方一致性数据
     *
     * @param name
     */
    @Override
    public void syncYzxData(String name, String qyId, String batch, RequestAttributes attributes) {
        List<String> carTypeList = certHgzPrintEODao.getDeclareVehicleType(qyId, batch);
        if (CollectionUtils.isEmpty(carTypeList)) {
            return;
        }
        long t1 = System.currentTimeMillis();
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) attributes;
        String sessionId = requestAttributes.getRequest().getRequestedSessionId();
        Object usermap = requestAttributes.getRequest().getSession().getAttribute("usermap");
        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(
                8,
                12,
                60,
                TimeUnit.SECONDS,
                new ArrayBlockingQueue<>(50),
                new TheadFactoryName(qyId));
        List<FactoryInfoVo> factoryInfoVos = this.baseMapper.getFactoryInfo(name);
        if (CollectionUtils.isEmpty(factoryInfoVos)) {
            log.error("TT_FACTORY表数据不存在：FACTORY_NAME=" + name);
            throw new BusinessException("企业名称在官方一致性数据中不存在：" + name);
        }
        Set<String> cocIdList = this.lambdaQuery().select(YzxBaseDataEO::getCocId).eq(YzxBaseDataEO::getQyId, qyId).list().stream().map(YzxBaseDataEO::getCocId).collect(Collectors.toSet());
        for (FactoryInfoVo factoryInfoVo : factoryInfoVos) {
            if (StringUtils.isBlank(factoryInfoVo.getUserAccount())) {
                log.error("TT_FACTORY.USER_ACCOUNT为空");
                throw new BusinessException("企业用户不存在");
            }
            List<CocInfoamtionVo> cocInfoamtionList = this.baseMapper.getCocInfoamtionList(factoryInfoVo.getFactoryId(), carTypeList);
            // 只插入，不更新
            Map<String, CocInfoamtionVo> cocInfoamtionVoMap = cocInfoamtionList.parallelStream().filter(x -> !cocIdList.contains(x.getCocId())).collect(Collectors.toMap(CocInfoamtionVo::getId, CocInfoamtionVo -> CocInfoamtionVo));
            if (cocInfoamtionVoMap.isEmpty()) {
                continue;
            }
            List<String> idList = new ArrayList<>(cocInfoamtionVoMap.keySet());

            // 一次处理500条数据，300条和1000条所花费的时间都比500多
            Lists.partition(idList, 500).forEach(x -> {
                threadPoolExecutor.execute(() -> {
                    initRequestAttributes(sessionId, usermap);
                    List<YzxBaseDataEO> baseTemplateInfos = this.baseMapper.getBaseTemplateInfo(x, factoryInfoVo.getUserAccount());
                    List<YzxBaseDataEO> yzxBaseDataEOS = new ArrayList<>();
                    baseTemplateInfos.forEach(info -> {
                        CocInfoamtionVo cocInfoamtionVo = cocInfoamtionVoMap.get(info.getId());
                        YzxBaseDataEO yzxBaseDataEO = new YzxBaseDataEO();
                        BeanUtils.copyProperties(info, yzxBaseDataEO);
                        yzxBaseDataEO.setQyId(qyId)
                                .setCarTypeSeq(cocInfoamtionVo.getCarTypeSeq())
                                .setCarCell(cocInfoamtionVo.getCarCell())
                                .setCarType(cocInfoamtionVo.getCarType())
                                .setCocId(cocInfoamtionVo.getCocId())
                                .setVehicleType(cocInfoamtionVo.getCocAutoType())
                                .setCocAutoState(cocInfoamtionVo.getCocAutoState())
                                .setAutoType(cocInfoamtionVo.getAutoType())
                                .setCocAutoFuelConsump(cocInfoamtionVo.getCocAutoFuelConsump())
                                .setCocAutoEffectiveDate(cocInfoamtionVo.getCocAutoEffectiveDate())
                                .setCocAutoStyle(cocInfoamtionVo.getCocAutoStyle())
                                .setCocFactorId(cocInfoamtionVo.getCocFactorId())
                                .setConfirmStatus(ConfirmStatus.NOT_CONFIRM.getValue())
                                .setCertificationPrincipalName(info.getCocApplyName());
                        yzxBaseDataEOS.add(yzxBaseDataEO);
                    });
                    if (CollectionUtils.isNotEmpty(yzxBaseDataEOS)) {
                        this.saveOrUpdateBatch(yzxBaseDataEOS);
                    }
                });
            });
        }
        threadPoolExecutor.shutdown();
        log.info("同步官方一致性数据完成，用时：" + (System.currentTimeMillis() - t1) / 1000 + "秒（qyId=" + qyId + "）;");
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String importYzxData(List<YzxTemplateInfoVo> templateInfoVos, String qyId) {
        // Vin编号和一致性证书编号是否重复
        Set<String> yzxCertnoList = templateInfoVos.stream().map(YzxTemplateInfoVo::getYzxCertno).collect(Collectors.toSet());
        Set<String> vinList = templateInfoVos.stream().map(YzxTemplateInfoVo::getVin).collect(Collectors.toSet());
        if (vinList.size() != templateInfoVos.size()) {
            throw new BusinessException("车辆识别码重复：");
        }
        // 校验一致性证书编号是否存在
        List<YzxBaseDataEO> existBaseDataEOS = this.lambdaQuery().in(YzxBaseDataEO::getCocId, yzxCertnoList).list();
        List<String> existYzxCertnoList = CollectionUtils.isEmpty(existBaseDataEOS) ? new ArrayList<>() : existBaseDataEOS.stream().map(YzxBaseDataEO::getCocId).collect(Collectors.toList());
        List<String> notExistYzxCertnoList = yzxCertnoList.stream().filter(x -> {
            return !existYzxCertnoList.contains(x);
        }).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(notExistYzxCertnoList)) {
            throw new BusinessException("COC证书编号在基础数据中不存在：" + notExistYzxCertnoList);
        }
        List<String> notConfirmedCocIds = new ArrayList<>();
        List<String> notBelongsSeleIds = new ArrayList<>();
        for (YzxBaseDataEO baseDataEO : existBaseDataEOS) {
            if (!ConfirmStatus.CONFIRMED.getValue().equals(baseDataEO.getConfirmStatus())) {
                notConfirmedCocIds.add(baseDataEO.getCocId());
            }
            if (!qyId.equals(baseDataEO.getQyId())) {
                notBelongsSeleIds.add(baseDataEO.getCocId());
            }
        }
        if (CollectionUtils.isNotEmpty(notConfirmedCocIds)) {
            throw new BusinessException("请先确认一致性基础数据后再导入：" + notConfirmedCocIds);
        }
        if (CollectionUtils.isNotEmpty(notBelongsSeleIds)) {
            throw new BusinessException("部分一致性编号不属于贵企业，如有疑问请联系客服人员！" + notConfirmedCocIds);
        }

        List<VehicleCertEO> vehicleCertEOS = vehicleCertEOService.lambdaQuery().eq(VehicleCertEO::getQyId, qyId).in(VehicleCertEO::getVin, vinList).list();
        Map<String, VehicleCertEO> vehicleCertEOVinMap = vehicleCertEOS.stream().collect(Collectors.toMap(VehicleCertEO::getVin, VehicleCertEO -> VehicleCertEO));

        // 获取已绑定的一致性基础数据
        List<String> vehicleCertIdList = vehicleCertEOS.stream().map(VehicleCertEO::getId).collect(Collectors.toList());
        Map<String, YzxTemplateDataEO> bindYzxTemplateDataEOMap = CollectionUtils.isEmpty(vehicleCertIdList) ? new HashMap<>() : yzxTemplateDataEOService.listByIds(vehicleCertIdList).stream().collect(Collectors.toMap(YzxTemplateDataEO::getId, YzxTemplateDataEO -> YzxTemplateDataEO));

        Map<String, YzxBaseDataEO> baseDataEOMap = existBaseDataEOS.stream().collect(Collectors.toMap(YzxBaseDataEO::getCocId, YzxBaseDataEO -> YzxBaseDataEO));

        List<VehicleCertEO> vehicleCertEOList = new ArrayList<>();
        List<YzxTemplateDataEO> yzxTemplateDataEOS = new ArrayList<>();
        Date sysDate = new Date();
        List<String> confirmVinNoList = new ArrayList<>();
        StringBuilder yzxCertnMsg = new StringBuilder();
        List<String> coverList = new ArrayList();
        String userId = UserUtil.getCurrentUserId();
        for (YzxTemplateInfoVo templateInfoVo : templateInfoVos) {
            VehicleCertEO vehicleCertEO = vehicleCertEOVinMap.get(templateInfoVo.getVin());
            YzxTemplateDataEO yzxTemplateDataEO = null;
            if (vehicleCertEO == null) {
                vehicleCertEO = new VehicleCertEO();
                vehicleCertEO.setId(UUID.randomUUID())
                        .setQyId(qyId)
                        .setVin(templateInfoVo.getVin())
                        .setCreateTime(sysDate)
                        .setCreator(userId);
            } else {
                yzxTemplateDataEO = bindYzxTemplateDataEOMap.get(vehicleCertEO.getId());
                // 已确认的VIN编号不处理并提示
                if (yzxTemplateDataEO != null && ConfirmStatus.CONFIRMED.getValue().equals(yzxTemplateDataEO.getConfirmStatus())) {
                    confirmVinNoList.add(vehicleCertEO.getVin());
                    continue;
                }
                // 导入前后一致性证书编号不一致的给出提示
                if (!templateInfoVo.getYzxCertno().equals(vehicleCertEO.getYzxCertno())) {
                    yzxCertnMsg.append("{车辆识别码：").append(vehicleCertEO.getVin()).append("，导入前COC证书编号：").append(vehicleCertEO.getYzxCertno())
                            .append("，导入后COC证书编号：").append(templateInfoVo.getYzxCertno()).append("}，");
                }
                coverList.add(vehicleCertEO.getVin());
            }
            vehicleCertEO.setYzxCertno(templateInfoVo.getYzxCertno())
                    .setFdjNo(templateInfoVo.getFdjNo())
                    .setVehicleColor(templateInfoVo.getVehicleColor())
                    .setProduceDate(templateInfoVo.getProduceDate())
                    .setUpdateTime(sysDate)
                    .setUpdater(userId);
            vehicleCertEOList.add(vehicleCertEO);

            if (yzxTemplateDataEO == null) {
                yzxTemplateDataEO = new YzxTemplateDataEO();
            }
            YzxBaseDataEO yzxBaseDataEO = baseDataEOMap.get(vehicleCertEO.getYzxCertno());
            if (!ConfirmStatus.CONFIRMED.getValue().equals(yzxBaseDataEO.getConfirmStatus())) {
                throw new BusinessException("COC证书编号为" + yzxBaseDataEO.getCocId() + "的基础数据未确认");
            }
            BeanUtils.copyProperties(yzxBaseDataEO, yzxTemplateDataEO);
            yzxTemplateDataEO.setId(vehicleCertEO.getId())
                    .setYzxPageNo(syxySerialNumberEOService.getNextSerialNumber(qyId, SerialNumberType.YZXZSDY))
                    .setDriveNo(templateInfoVo.getFdjNo())
                    .setDriveNoHh(templateInfoVo.getQddjNo())
                    .setConfirmStatus(ConfirmStatus.NOT_CONFIRM.getValue())
                    .setPrintStatus(PrintStatus.NOT_PRINT.getValue())
                    .setCreateTime(sysDate)
                    .setCreator(userId)
                    .setUpdateTime(sysDate)
                    .setUpdater(userId);
            yzxTemplateDataEOS.add(yzxTemplateDataEO);
        }
        if (CollectionUtils.isNotEmpty(vehicleCertEOList)) {
            vehicleCertEOService.saveOrUpdateBatch(vehicleCertEOList);
        }
        if (CollectionUtils.isNotEmpty(yzxTemplateDataEOS)) {
            yzxTemplateDataEOService.saveOrUpdateBatch(yzxTemplateDataEOS);
        }
        StringBuilder okMsg = new StringBuilder();
        okMsg.append("导入完成。");
        if (CollectionUtils.isNotEmpty(confirmVinNoList)) {
            okMsg.append("已确认的车辆识别码信息无法修改：").append(confirmVinNoList).append("。");
        }
        if (yzxCertnMsg.length() > 0) {
            okMsg.append("导入前后COC证书编号不一致（已覆盖）：[").append(yzxCertnMsg.deleteCharAt(yzxCertnMsg.length() - 1)).append("]。");
        }
        if (coverList.size() > 0) {
            okMsg.append("导入的发动机、车辆颜色、制造日期信息已被覆盖：").append(coverList).append("。");
        }
        return okMsg.toString();
    }

    @Override
    public Map createWords(List<String> vehicleCertIdList, boolean isPdfView, boolean isFromDeclare) {
        Map res = new LinkedHashMap();
        Collection<VehicleCertEO> vehicleCertEOS = vehicleCertEOService.listByIds(vehicleCertIdList);
        Map<String, VehicleCertEO> vehicleCertEOMap = vehicleCertEOS.stream().collect(Collectors.toMap(VehicleCertEO::getId, VehicleCertEO -> VehicleCertEO));
        Map<String, YzxTemplateDataEO> templateDataEOMap = yzxTemplateDataEOService.listByIds(vehicleCertIdList).stream().collect(Collectors.toMap(YzxTemplateDataEO::getId, YzxTemplateDataEO -> YzxTemplateDataEO));
        TemplateType templateType = TemplateType.NONE;

        List<String> codeList = vehicleCertEOS.stream().map(x -> {
            return new StringBuilder().append(x.getYzxCertno()).append(StringUtils.SPACE).append(x.getVin()).toString();
        }).collect(Collectors.toList());
        Map qrMap = this.creatCOCQR(codeList);
        String qyId = UserUtil.getQyId();
        String date = DateUtils.dateToString(new Date(), DateUtils.YYYY_MM_DD_EN);
        String pathSuffix = new StringBuilder().append(qyId).append("/").append("YiZhiXing/").append(date).append("/").toString();
        try {
            for (String vehicleCertId : vehicleCertIdList) {
                VehicleCertEO vehicleCertEO = vehicleCertEOMap.get(vehicleCertId);
                if (vehicleCertEO == null) {
                    log.error("VEHICLE_CERT的数据丢失：id=" + vehicleCertId);
                    throw new BusinessException("一致性证书数据丢失");
                }
                YzxTemplateDataEO templateDataEO = templateDataEOMap.get(vehicleCertEO.getId());
                if (templateDataEO == null) {
                    log.error("YZX_TEMPLATE_DATA的数据丢失：id=" + vehicleCertEO.getId());
                    throw new BusinessException("一致性证书模板数据丢失");
                }
                if (PrintStatus.PRINTED.getValue().equals(templateDataEO.getPrintStatus()) && !isFromDeclare) {
                    throw new BusinessException("VIN编号为" + vehicleCertEO.getVin() + "的一致性证书已被打印！");
                }
                if (ConfirmStatus.NOT_CONFIRM.getValue().equals(templateDataEO.getConfirmStatus())) {
                    throw new BusinessException("VIN编号为" + vehicleCertEO.getVin() + "的一致性证书未确认，无法打印！");
                }
                String code = new StringBuilder().append(vehicleCertEO.getYzxCertno()).append(StringUtils.SPACE).append(vehicleCertEO.getVin()).toString();
                YzxPrintData yzxPrintData = new YzxPrintData();
                BeanUtils.copyProperties(templateDataEO, yzxPrintData);

                templateType = TemplateType.getByValue(templateDataEO.getTemplateType());

                String templatePath = ResourceUtil.getConfigByName("realpath") + templateType.getName();
                File templateFile = new File(templatePath);
                if (!templateFile.exists()) {
                    log.error(templateType.getName() + "打印模板不存在");
                    throw new BusinessException(templateType.getName() + "打印模板不存在");
                }
                // 车辆识别代号
                yzxPrintData.setVin(vehicleCertEO.getVin());
                // 车辆颜色
                yzxPrintData.setVehicleColor(vehicleCertEO.getVehicleColor());
                // 外廓尺寸（长/宽/高）（mm）
                yzxPrintData.setCarOutlineSize(new StringJoiner(CommConstant.SLASH_EN)
                        .add(StringUtils.defaultString(templateDataEO.getCarLength(), CommConstant.CHINESE_CHARACTER_NOT_HAVE))
                        .add(StringUtils.defaultString(templateDataEO.getCarWidth(), CommConstant.CHINESE_CHARACTER_NOT_HAVE))
                        .add(StringUtils.defaultString(templateDataEO.getCarHight(), CommConstant.CHINESE_CHARACTER_NOT_HAVE)).toString());
                // 货箱内部尺寸（长/宽/高）（ mm）
                yzxPrintData.setVehicleInternalSize(new StringJoiner(CommConstant.SLASH_EN)
                        .add(StringUtils.defaultString(templateDataEO.getVehicleLength(), CommConstant.CHINESE_CHARACTER_NOT_HAVE))
                        .add(StringUtils.defaultString(templateDataEO.getVehicleWidth(), CommConstant.CHINESE_CHARACTER_NOT_HAVE))
                        .add(StringUtils.defaultString(templateDataEO.getVehicleHight(), CommConstant.CHINESE_CHARACTER_NOT_HAVE)).toString());
                // 最大允许牵引质量（kg）
                String maxTractionQuality = StringUtils.EMPTY;
                if (!"不适用".equals(templateDataEO.getTrailerMaxWeight())) {
                    maxTractionQuality = templateDataEO.getTrailerMaxWeight();
                } else if (!"不适用".equals(templateDataEO.getTractorTrailer())) {
                    maxTractionQuality = templateDataEO.getTractorTrailer();
                } else if (!"不适用".equals(templateDataEO.getSemiTrailer())) {
                    maxTractionQuality = templateDataEO.getSemiTrailer();
                } else if (!"不适用".equals(templateDataEO.getIntermediateTrailer())) {
                    maxTractionQuality = templateDataEO.getIntermediateTrailer();
                }
                yzxPrintData.setMaxTractionQuality(maxTractionQuality);
                // 车辆制造日期
                yzxPrintData.setProduceDate(vehicleCertEO.getProduceDate());
                // 拆分轮胎规格，转换轮胎规格格式
                yzxPrintData.setCarTireSpecification(this.splitString(yzxPrintData.getCarTireSpecification()));
                // 排气污染物-CO
                yzxPrintData.setCoLiquid(templateDataEO.getCoLiquidWz());
                // 排气污染物-THC
                yzxPrintData.setThcLiquid(templateDataEO.getThcLiquid());
                // 排气污染物-NMHC
                yzxPrintData.setNmhcLiquid(templateDataEO.getNmhcLiquid());
                // 排气污染物-NOx
                yzxPrintData.setNoxLiquid(templateDataEO.getNoxLiquidWz());
                // 排气污染物-CH4
                yzxPrintData.setCh4Liquid(templateDataEO.getCh4Liquid());
                // 排气污染物-NH3
                yzxPrintData.setNh3Liquid(templateDataEO.getNh3Liquid());
                // 排气污染物-PM
                yzxPrintData.setPmLiquid(templateDataEO.getPmLiquid());
                // 排气污染物-PN
                yzxPrintData.setPnLiquid(templateDataEO.getPnLiquid());
                if (templateType == TemplateType.TEMPLATE_10) {
                    // 排气污染物-CO
                    yzxPrintData.setCoLiquid(templateDataEO.getCoLiquidBwz());
                    // 排气污染物-NOx
                    yzxPrintData.setNoxLiquid(templateDataEO.getNoxLiquidBwz());
                }
                WordprocessingMLPackage template = Docx4jUtil.getTemplate(templatePath);

                // 根据字符串生成二维码
                String text = String.valueOf(qrMap.get(code)); // 二维码内容
                int width = 90; // 二维码图片宽度
                int height = 90; // 二维码图片高度
                String format = "png";// 二维码的图片格式
                String cocQrPath = ResourceUtil.getConfigByName("COCQRPath");
                String savePath = cocQrPath + vehicleCertEO.getId() + ".png";
                GoogleQRCodeUtil.createQRCode(text, width, height, format, savePath);
                // 替换成二维码
                File qrPic = new File(savePath);
                if (qrPic.exists()) {
                    byte[] bytes = Docx4jUtil.convertImageToByteArray(qrPic);
                    Docx4jUtil.replaceImage(template, "${qr}", bytes, "", "");
                } else {
                    log.error("二维码图片生成失败:" + qrPic);
                    throw new BusinessException("二维码图片生成失败");
                }

                ObjectMapper objectMapper = new ObjectMapper();
                objectMapper.getSerializerProvider().setNullValueSerializer(new JsonSerializer<Object>() {
                    @Override
                    public void serialize(Object value, JsonGenerator jsonGenerator, SerializerProvider serializerProvider) throws IOException {
                        jsonGenerator.writeString(StringUtils.EMPTY);
                    }
                });
                HashMap<String, String> templateData = objectMapper.readValue(objectMapper.writeValueAsString(yzxPrintData), HashMap.class);
                templateData.entrySet().forEach(x -> {
                    if (StringUtils.EMPTY.equals(x.getValue())) {
                        x.setValue("/");
                    }
                });
                Docx4jUtil.variableReplace(template, templateData);

                String previewPath = ResourceUtil.getConfigByName("previewPath") + pathSuffix;
                // 预览文件保存路径
                File docxFile = new File(previewPath);
                if (!docxFile.exists()) {
                    docxFile.mkdirs();
                }
                String docxFileName = new StringBuilder("uniData_").append(vehicleCertEO.getId()).append(".docx").toString();
                String docxFilePath = previewPath + docxFileName;
                Docx4jUtil.writeDocxToStream(template, docxFilePath);

                if (isPdfView) {
                    String pdfFileName = new StringBuilder("uniData_").append(vehicleCertEO.getId()).append(".pdf").toString();
                    String pdfFilePath = previewPath + pdfFileName;
                    PDFUtils2.wordToPdf(new File(docxFilePath), new File(pdfFilePath), wordToPdfUrl);
                    res.put(vehicleCertId, pathSuffix + pdfFileName);
                    continue;
                }
                res.put(vehicleCertId, pathSuffix + docxFileName);
            }
        } catch (FileNotFoundException e) {
            log.error(templateType.getName() + "打印模板不存在");
            throw new BusinessException(templateType.getName() + "打印模板不存在");
        } catch (BusinessException e) {
            throw e;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return res;
    }

    @Override
    public CocInfoamtionVo getCocInfoamtion(String cocId) {
        return this.baseMapper.getCocInfoamtion(cocId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchBaseCancel(List<String> idList) {
        Collection<YzxBaseDataEO> yzxBaseDataEOS = this.listByIds(idList);
        List<String> existIdList = new ArrayList<>();
        List<String> cocIdList = new ArrayList<>();
        for (YzxBaseDataEO dataEO : yzxBaseDataEOS) {
            existIdList.add(dataEO.getId());
            cocIdList.add(dataEO.getCocId());
        }
        String qyId = UserUtil.getQyId();
        List<VehicleCertEO> vehicleCertEOList = CollectionUtils.isEmpty(cocIdList) ? new ArrayList<>() :
                vehicleCertEOService.lambdaQuery().in(VehicleCertEO::getYzxCertno, cocIdList).eq(VehicleCertEO::getQyId, qyId).list();
        List<String> vehicleCertIdList = new ArrayList<>();
        vehicleCertEOList.forEach(x -> vehicleCertIdList.add(x.getId()));

        List<YzxTemplateDataEO> yzxTemplateDataEOList = CollectionUtils.isEmpty(vehicleCertIdList) ? new ArrayList<>() :
                yzxTemplateDataEOService.lambdaQuery().in(YzxTemplateDataEO::getId, vehicleCertIdList).list();
        List<String> updateIdList = new ArrayList<>();
        for (YzxTemplateDataEO yzxTemplateDataEO : yzxTemplateDataEOList) {
            if (DeclareStatus.DECLARED.getValue().equals(yzxTemplateDataEO.getDeclareStatus())) {
                throw new BusinessException("存在已上传的一致性证书，无法取消");
            }
            yzxTemplateDataEOList.forEach(x -> updateIdList.add(x.getId()));
        }

        if (CollectionUtils.isNotEmpty(updateIdList)) {
            yzxTemplateDataEOService.removeByIds(updateIdList);
            vehicleCertEOService.update(new UpdateWrapper<VehicleCertEO>().set(VehicleCertEO.VEHICLE_CERT_YZX_CERTNO, null).in(VehicleCertEO.VEHICLE_CERT_ID, updateIdList));
        }
        this.update(new UpdateWrapper<YzxBaseDataEO>().set(YzxBaseDataEO.CONFIRM_STATUS, ConfirmStatus.NOT_CONFIRM.getValue())
                .set(YzxBaseDataEO.UPDATE_TIME, new Date())
                .set(YzxBaseDataEO.UPDATER, UserUtil.getCurrentUserId())
                .in(YzxBaseDataEO.ID, idList)
                .eq(YzxBaseDataEO.QY_ID, qyId));
    }

    @Override
    public String splitString(String tireSize) {
        StringBuffer str = new StringBuffer();
        if (org.apache.commons.lang.StringUtils.isNotEmpty(tireSize) && tireSize.contains("@")) {
            int num = 0;
            String[] Car_Tire_Specifications = tireSize.split("@");
            for (int i = 0; i < Car_Tire_Specifications.length; i++) {
                num++;
                str.append("第" + num + "轴" + Car_Tire_Specifications[i]);
            }
            return str.toString();
        }
        return tireSize;
    }

    /**
     * 调用生成二维码控件
     *
     * @return
     */
    private Map creatCOCQR(List<String> args) {
        RequestConfig requestConfig = RequestConfig.custom()
                .setSocketTimeout(5000)// 读取超时
                .setConnectTimeout(5000)// 连接超时
                .build();
        CloseableHttpClient httpClient = HttpClients.custom().setDefaultRequestConfig(requestConfig).build();

        HttpPost httpPost = new HttpPost(batchEncryptUrl);
        CloseableHttpResponse response = null;
        Map result = null;
        try {
            StringEntity httpEntity = new StringEntity(JSONUtil.toJsonStr(args), "utf-8");
            httpEntity.setContentType("application/json;charset=utf-8");
            httpPost.setEntity(httpEntity);
            response = httpClient.execute(httpPost);
            int statecode = response.getStatusLine().getStatusCode();
            if (statecode == 200) {
                HttpEntity httpentity = response.getEntity();
                if (httpentity != null) {
                    String resStr = EntityUtils.toString(httpentity);
                    log.info("二维码控件返回结果：", resStr);
                    Map map = JSON.parseObject(resStr, Map.class);
                    int status = (int) map.get("status");
                    if (status == 200) {
                        result = (Map) map.getOrDefault("data", new HashMap<>());
                        return result;
                    }
                    throw new BusinessException("二维码生成失败");
                }
            }
            log.error("二维码控件返回结果response：", response);
            throw new BusinessException("二维码生成失败");
        } catch (Exception e) {
            log.error("二维码服务调用失败", e);
            throw new BusinessException("二维码生成失败");
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
            try {
                httpPost.abort();
            } catch (Exception e) {
                e.printStackTrace();
            }
            try {
                httpClient.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    private void initRequestAttributes(String sessionId, Object usermap) {
        StandardManager standardManager = new StandardManager();
        StandardSession standardSession = new StandardSession(standardManager);
        standardSession.setValid(true);
        standardSession.setId(sessionId, false);
        standardManager.add(standardSession);
        StandardContext standardContext = new StandardContext();
        standardContext.setManager(standardManager);
        Request request = new Request();
        request.getMappingData().context = standardContext;
        request.setRequestedSessionId(sessionId);
        RequestFacade requestFacade = new RequestFacade(request);
        requestFacade.getSession().setAttribute("usermap", usermap);
        RequestContextHolder.setRequestAttributes(new ServletRequestAttributes(requestFacade));
    }
}
