package com.adc.da.vdaims.service.impl;

import com.adc.da.util.utils.BeanUtils;
import com.adc.da.util.utils.StringUtils;
import com.adc.da.vdaims.entity.VehicleCertEO;
import com.adc.da.vdaims.entity.VehicleCertTemplateEO;
import com.adc.da.vdaims.entity.YzxTemplateDataEO;
import com.adc.da.vdaims.dao.YzxTemplateDataEODao;
import com.adc.da.vdaims.enums.ConfirmStatus;
import com.adc.da.vdaims.enums.DeclareStatus;
import com.adc.da.vdaims.enums.PrintStatus;
import com.adc.da.vdaims.enums.TemplateType;
import com.adc.da.vdaims.exception.BusinessException;
import com.adc.da.vdaims.handler.YzxTemplateHandler;
import com.adc.da.vdaims.service.IVehicleCertEOService;
import com.adc.da.vdaims.service.IYzxBaseDataEOService;
import com.adc.da.vdaims.service.IYzxTemplateDataEOService;
import com.adc.da.vdaims.vo.CocInfoamtionVo;
import com.adc.da.vdaims.vo.BaseTemplateVo;
import com.adc.da.vdaims.vo.VehicleCertYzxVo;
import com.adc.da.vdaims.vo.YzxTemplateExcelVo;
import com.adc.da.vdaims.vo.YzxDeclareExcelVo;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 一致性模板数据 服务实现类
 * </p>
 * 作者 gen
 * 日期 2021-09-01
 */
@Service
public class YzxTemplateDataEOServiceImpl extends ServiceImpl<YzxTemplateDataEODao, YzxTemplateDataEO> implements IYzxTemplateDataEOService {
    @Autowired
    private IVehicleCertEOService vehicleCertEOService;

    @Override
    public BaseTemplateVo getTemplate(String vehicleCertId, String templateType) {
        VehicleCertEO vehicleCertEO = vehicleCertEOService.getById(vehicleCertId);
        if (vehicleCertEO == null) {
            log.error("VEHICLE_CERT的数据丢失：id="+vehicleCertId);
            throw new BusinessException("一致性证书数据丢失");
        }
        YzxTemplateDataEO templateDataEO = this.getById(vehicleCertEO.getId());
        if (templateDataEO == null) {
            log.error("YZX_TEMPLATE_DATA的数据丢失：id="+vehicleCertEO.getId());
            throw new BusinessException("一致性证书模板数据丢失");
        }
        if (StringUtils.isBlank(templateDataEO.getTemplateType())) {
            templateDataEO.setTemplateType(templateType);
            this.updateById(templateDataEO);
        }
        BaseTemplateVo templateVo = null;
        switch (TemplateType.getByValue(templateDataEO.getTemplateType())) {
            case TEMPLATE_1:
                templateVo = YzxTemplateHandler.getYzxTemplate_1(templateDataEO, vehicleCertEO);
                break;
            case TEMPLATE_2:
                templateVo = YzxTemplateHandler.getYzxTemplate_2(templateDataEO, vehicleCertEO);
                break;
            case TEMPLATE_3:
                templateVo = YzxTemplateHandler.getYzxTemplate_3(templateDataEO, vehicleCertEO);
                break;
            case TEMPLATE_4:
                templateVo = YzxTemplateHandler.getYzxTemplate_4(templateDataEO, vehicleCertEO);
                break;
            case TEMPLATE_5:
                templateVo = YzxTemplateHandler.getYzxTemplate_5(templateDataEO, vehicleCertEO);
                break;
            case TEMPLATE_6:
                templateVo = YzxTemplateHandler.getYzxTemplate_6(templateDataEO, vehicleCertEO);
                break;
            case TEMPLATE_7:
                templateVo = YzxTemplateHandler.getYzxTemplate_7(templateDataEO, vehicleCertEO);
                break;
            case TEMPLATE_8:
                templateVo = YzxTemplateHandler.getYzxTemplate_8(templateDataEO, vehicleCertEO);
                break;
            case TEMPLATE_9:
                templateVo = YzxTemplateHandler.getYzxTemplate_9(templateDataEO, vehicleCertEO);
                break;
            case TEMPLATE_10:
                templateVo = YzxTemplateHandler.getYzxTemplate_10(templateDataEO, vehicleCertEO);
                break;
            case TEMPLATE_11:
                templateVo = YzxTemplateHandler.getYzxTemplate_11(templateDataEO, vehicleCertEO);
                break;
            case TEMPLATE_12:
                templateVo = YzxTemplateHandler.getYzxTemplate_12(templateDataEO, vehicleCertEO);
                break;
            case TEMPLATE_13:
                templateVo = YzxTemplateHandler.getYzxTemplate_13(templateDataEO, vehicleCertEO);
                break;
            default:
                log.error("一致性模板类型不存在：TemplateType="+templateDataEO.getTemplateType());
                throw new BusinessException("一致性证书模板不存在");
        }
        return templateVo;
    }

    @Override
    public List<YzxTemplateExcelVo> getYzxTemplateExcelVoList(Wrapper<VehicleCertYzxVo> queryWrapper, String qyId) {
        List<YzxTemplateExcelVo> yzxTemplateExcelVoList = new ArrayList<>();
        List<VehicleCertTemplateEO> vehicleCertTemplateEOS = this.baseMapper.selectVehicleCertTemplateList(queryWrapper, qyId);
        vehicleCertTemplateEOS.forEach(x->{
            YzxTemplateExcelVo excelVo = new YzxTemplateExcelVo();
            BeanUtils.copyProperties(x, excelVo);
            // 排气污染物-CCC 认证所依据的标准号及实施阶段
            excelVo.setAuthenticationReference(x.getAuthenticationReferenceWz());
            // 排气污染物-CO
            excelVo.setCoLiquid(x.getCoLiquidWz());
            // 排气污染物-THC
            excelVo.setThcLiquid(x.getThcLiquid());
            // 排气污染物-NMHC
            excelVo.setNmhcLiquid(x.getNmhcLiquid());
            // 排气污染物-NOx
            excelVo.setNoxLiquid(x.getNoxLiquidWz());
            // 排气污染物-N2O
            excelVo.setN2oLiquid(x.getN2oLiquid());
            // 排气污染物-CH4
            excelVo.setCh4Liquid(x.getCh4Liquid());
            // 排气污染物-NH3
            excelVo.setNh3Liquid(x.getNh3Liquid());
            switch (TemplateType.getByValue(x.getTemplateType())){
                case TEMPLATE_6:
                case TEMPLATE_9:
                    // 排气污染物-CO
                    excelVo.setCoLiquid(x.getCoLiquidBwz());
                    // 排气污染物-NOx
                    excelVo.setNoxLiquid(x.getNoxLiquidBwz());
                    break;
                default:
                    break;
            }
            // 最大允许牵引质量（kg)
            String maxTractionQuality = StringUtils.EMPTY;
            if (!"不适用".equals(x.getTrailerMaxWeight())) {
                maxTractionQuality = x.getTrailerMaxWeight();
            } else if (!"不适用".equals(x.getTractorTrailer())) {
                maxTractionQuality = x.getTractorTrailer();
            } else if (!"不适用".equals(x.getSemiTrailer())) {
                maxTractionQuality = x.getSemiTrailer();
            } else if (!"不适用".equals(x.getIntermediateTrailer())) {
                maxTractionQuality = x.getIntermediateTrailer();
            }
            excelVo.setMaxTractionQuality(maxTractionQuality);
            excelVo.setConfirmStatus(ConfirmStatus.getByValue(x.getConfirmStatus()).getName());
            excelVo.setPrintStatus(PrintStatus.getByValue(x.getPrintStatus()).getName());
            yzxTemplateExcelVoList.add(excelVo);
        });
        return yzxTemplateExcelVoList;
    }

    @Override
    public List<YzxDeclareExcelVo> getYzxDeclareExcelVoList(Wrapper<VehicleCertYzxVo> queryWrapper, String qyId) {
        List<YzxDeclareExcelVo> yzxDeclareExcelVoList = new ArrayList<>();
        List<VehicleCertTemplateEO> vehicleCertTemplateEOS = this.baseMapper.selectVehicleCertTemplateList(queryWrapper, qyId);
        vehicleCertTemplateEOS.forEach(x->{
            YzxDeclareExcelVo excelVo = new YzxDeclareExcelVo();
            BeanUtils.copyProperties(x, excelVo);
            // 排气污染物-CCC 认证所依据的标准号及实施阶段
            excelVo.setAuthenticationReference(x.getAuthenticationReferenceWz());
            // 排气污染物-CO
            excelVo.setCoLiquid(x.getCoLiquidWz());
            // 排气污染物-THC
            excelVo.setThcLiquid(x.getThcLiquid());
            // 排气污染物-NMHC
            excelVo.setNmhcLiquid(x.getNmhcLiquid());
            // 排气污染物-NOx
            excelVo.setNoxLiquid(x.getNoxLiquidWz());
            // 排气污染物-N2O
            excelVo.setN2oLiquid(x.getN2oLiquid());
            // 排气污染物-CH4
            excelVo.setCh4Liquid(x.getCh4Liquid());
            // 排气污染物-NH3
            excelVo.setNh3Liquid(x.getNh3Liquid());
            switch (TemplateType.getByValue(x.getTemplateType())){
                case TEMPLATE_6:
                case TEMPLATE_9:
                    // 排气污染物-CO
                    excelVo.setCoLiquid(x.getCoLiquidBwz());
                    // 排气污染物-NOx
                    excelVo.setNoxLiquid(x.getNoxLiquidBwz());
                    break;
                default:
                    break;
            }
            // 最大允许牵引质量（kg)
            String maxTractionQuality = StringUtils.EMPTY;
            if (!"不适用".equals(x.getTrailerMaxWeight())) {
                maxTractionQuality = x.getTrailerMaxWeight();
            } else if (!"不适用".equals(x.getTractorTrailer())) {
                maxTractionQuality = x.getTractorTrailer();
            } else if (!"不适用".equals(x.getSemiTrailer())) {
                maxTractionQuality = x.getSemiTrailer();
            } else if (!"不适用".equals(x.getIntermediateTrailer())) {
                maxTractionQuality = x.getIntermediateTrailer();
            }
            excelVo.setMaxTractionQuality(maxTractionQuality);
            excelVo.setDeclareStatus(DeclareStatus.getByValue(x.getDeclareStatus()).getName());
            yzxDeclareExcelVoList.add(excelVo);
        });
        return yzxDeclareExcelVoList;
    }
}
